from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.types import UNSET, Unset

T = TypeVar("T", bound="PatchDataSourceFieldRequest")


@_attrs_define
class PatchDataSourceFieldRequest:
    """Example:
        {'name': 'Population', 'isRequired': True, 'defaultValue': 130000}

    Attributes:
        name (Union[None, Unset, str]): The name of the field. It must be unique within the data source.
        is_required (Union[None, Unset, bool]): Whether the field is required. If true, the field must be filled in when
            creating a data source item.
        is_multiple_lines (Union[None, Unset, bool]): Whether the field is multiple lines. If true, the field will be
            rendered as a text area.
        default_value (Union[None, Unset, str]): The default value of the field. If specified, the field will be pre-
            filled with this value when creating a data source item.
    """

    name: None | Unset | str = UNSET
    is_required: None | Unset | bool = UNSET
    is_multiple_lines: None | Unset | bool = UNSET
    default_value: None | Unset | str = UNSET

    def to_dict(self) -> dict[str, Any]:
        name: None | Unset | str
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        is_required: None | Unset | bool
        if isinstance(self.is_required, Unset):
            is_required = UNSET
        else:
            is_required = self.is_required

        is_multiple_lines: None | Unset | bool
        if isinstance(self.is_multiple_lines, Unset):
            is_multiple_lines = UNSET
        else:
            is_multiple_lines = self.is_multiple_lines

        default_value: None | Unset | str
        if isinstance(self.default_value, Unset):
            default_value = UNSET
        else:
            default_value = self.default_value

        field_dict: dict[str, Any] = {}

        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if is_required is not UNSET:
            field_dict["isRequired"] = is_required
        if is_multiple_lines is not UNSET:
            field_dict["isMultipleLines"] = is_multiple_lines
        if default_value is not UNSET:
            field_dict["defaultValue"] = default_value

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)

        def _parse_name(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_is_required(data: object) -> None | Unset | bool:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | bool", data)

        is_required = _parse_is_required(d.pop("isRequired", UNSET))

        def _parse_is_multiple_lines(data: object) -> None | Unset | bool:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | bool", data)

        is_multiple_lines = _parse_is_multiple_lines(d.pop("isMultipleLines", UNSET))

        def _parse_default_value(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        default_value = _parse_default_value(d.pop("defaultValue", UNSET))

        patch_data_source_field_request = cls(
            name=name,
            is_required=is_required,
            is_multiple_lines=is_multiple_lines,
            default_value=default_value,
        )

        return patch_data_source_field_request



from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Self

from templafy.types import UNSET, Unset

T = TypeVar("T", bound="DataSourceLanguageItemField")


@_attrs_define
class DataSourceLanguageItemField:
    """Example:
        {'dataSourceFieldId': 6, 'type': 'language', 'cultureName': 'en-GB'}

    Attributes:
        type_ (Union[None, str]): Data source item field type.
        data_source_field_id (Union[Unset, int]): Unique data source field identifier.
        culture_name (Union[None, Unset, str]): Language item field culture name.
    """

    type_: None | str
    data_source_field_id: Unset | int = UNSET
    culture_name: None | Unset | str = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_: None | str
        type_ = self.type_

        data_source_field_id = self.data_source_field_id

        culture_name: None | Unset | str
        if isinstance(self.culture_name, Unset):
            culture_name = UNSET
        else:
            culture_name = self.culture_name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type_,
            }
        )
        if data_source_field_id is not UNSET:
            field_dict["dataSourceFieldId"] = data_source_field_id
        if culture_name is not UNSET:
            field_dict["cultureName"] = culture_name

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)

        def _parse_type_(data: object) -> None | str:
            if data is None:
                return data
            return cast("None | str", data)

        type_ = _parse_type_(d.pop("type"))

        data_source_field_id = d.pop("dataSourceFieldId", UNSET)

        def _parse_culture_name(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        culture_name = _parse_culture_name(d.pop("cultureName", UNSET))

        data_source_language_item_field = cls(
            type_=type_,
            data_source_field_id=data_source_field_id,
            culture_name=culture_name,
        )

        data_source_language_item_field.additional_properties = d
        return data_source_language_item_field

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Self

from templafy.types import UNSET, Unset

T = TypeVar("T", bound="LanguageFieldSchema")


@_attrs_define
class LanguageFieldSchema:
    """Example:
        {'id': 4, 'name': 'PreferredLanguage', 'type': 'language', 'isRequired': False, 'isLocked': False,
            'defaultValue': 'German'}

    Attributes:
        type_ (str): Data source field schema type.
        id (int): Unique data source field schema identifier
        name (str): Data source field schema name. It must be unique within the data source.
        is_locked (bool): Value indicating whether data source schema is locked. If true, the field cannot be deleted or
            modified.
        is_required (bool): Whether the field is required. If true, the field must be filled in when creating a data
            source item.
        default_value (Union[None, Unset, str]): The default value of the field. If specified, the field will be pre-
            filled with this value when creating a data source item.
    """

    type_: str
    id: int
    name: str
    is_locked: bool
    is_required: bool
    default_value: None | Unset | str = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        id = self.id

        name = self.name

        is_locked = self.is_locked

        is_required = self.is_required

        default_value: None | Unset | str
        if isinstance(self.default_value, Unset):
            default_value = UNSET
        else:
            default_value = self.default_value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type_,
                "id": id,
                "name": name,
                "isLocked": is_locked,
                "isRequired": is_required,
            }
        )
        if default_value is not UNSET:
            field_dict["defaultValue"] = default_value

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)
        type_ = d.pop("type")

        id = d.pop("id")

        name = d.pop("name")

        is_locked = d.pop("isLocked")

        is_required = d.pop("isRequired")

        def _parse_default_value(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        default_value = _parse_default_value(d.pop("defaultValue", UNSET))

        language_field_schema = cls(
            type_=type_,
            id=id,
            name=name,
            is_locked=is_locked,
            is_required=is_required,
            default_value=default_value,
        )

        language_field_schema.additional_properties = d
        return language_field_schema

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.models.lock_reason import LockReason
from templafy.types import UNSET, Unset

if TYPE_CHECKING:
    from templafy.models.dependency import Dependency
from templafy.models.dependency import Dependency

T = TypeVar("T", bound="DataSourceObjectLockedProblemDetails")


@_attrs_define
class DataSourceObjectLockedProblemDetails:
    """The reason the resource is locked with an optional array of dependencies. Dependencies array is populated only when
    lockedReason is hardDependency and contains a maximum of 50 items per sourceEntityType

    Example:
            {'title': 'Locked', 'detail': 'The data source item cannot be deleted because it has been used by another
                resource.', 'status': 423, 'traceId': 'd61f7ce-cccb-4e5b-8727-3b68a61a0559', 'lockReason': 'hardDependency',
                'dependencies': [{'sourceEntityType': 'dataSourceItem', 'sourceEntityId': '1031936131644784655', 'description':
                "There is a dependency from 'DataSourceItem'."}]}

    Attributes:
            lock_reason (Union[Unset, LockReason]): The reason the resource is locked. It is either because the resource is
                depended upon by another resource or the resource has restricted access and cannot be modified.
            dependencies (Union[None, Unset, list['Dependency']]):
            type_ (Union[None, Unset, str]):
            title (Union[None, Unset, str]):
            status (Union[None, Unset, int]):
            detail (Union[None, Unset, str]):
            instance (Union[None, Unset, str]):
            trace_id (Union[None, Unset, str]):
    """

    lock_reason: Unset | LockReason = UNSET
    dependencies: None | Unset | list["Dependency"] = UNSET
    type_: None | Unset | str = UNSET
    title: None | Unset | str = UNSET
    status: None | Unset | int = UNSET
    detail: None | Unset | str = UNSET
    instance: None | Unset | str = UNSET
    trace_id: None | Unset | str = UNSET

    def to_dict(self) -> dict[str, Any]:
        lock_reason: Unset | str = UNSET
        if not isinstance(self.lock_reason, Unset):
            lock_reason = self.lock_reason.value

        dependencies: None | Unset | list[dict[str, Any]]
        if isinstance(self.dependencies, Unset):
            dependencies = UNSET
        elif isinstance(self.dependencies, list):
            dependencies = []
            for dependencies_type_0_item_data in self.dependencies:
                dependencies_type_0_item = dependencies_type_0_item_data.to_dict()
                dependencies.append(dependencies_type_0_item)

        else:
            dependencies = self.dependencies

        type_: None | Unset | str
        if isinstance(self.type_, Unset):
            type_ = UNSET
        else:
            type_ = self.type_

        title: None | Unset | str
        if isinstance(self.title, Unset):
            title = UNSET
        else:
            title = self.title

        status: None | Unset | int
        if isinstance(self.status, Unset):
            status = UNSET
        else:
            status = self.status

        detail: None | Unset | str
        if isinstance(self.detail, Unset):
            detail = UNSET
        else:
            detail = self.detail

        instance: None | Unset | str
        if isinstance(self.instance, Unset):
            instance = UNSET
        else:
            instance = self.instance

        trace_id: None | Unset | str
        if isinstance(self.trace_id, Unset):
            trace_id = UNSET
        else:
            trace_id = self.trace_id

        field_dict: dict[str, Any] = {}

        field_dict.update({})
        if lock_reason is not UNSET:
            field_dict["lockReason"] = lock_reason
        if dependencies is not UNSET:
            field_dict["dependencies"] = dependencies
        if type_ is not UNSET:
            field_dict["type"] = type_
        if title is not UNSET:
            field_dict["title"] = title
        if status is not UNSET:
            field_dict["status"] = status
        if detail is not UNSET:
            field_dict["detail"] = detail
        if instance is not UNSET:
            field_dict["instance"] = instance
        if trace_id is not UNSET:
            field_dict["traceId"] = trace_id

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        # Top-level imports are used to avoid inline imports (PLC0415)
        d = dict(src_dict)
        _lock_reason = d.pop("lockReason", UNSET)
        lock_reason: Unset | LockReason
        if isinstance(_lock_reason, Unset):
            lock_reason = UNSET
        else:
            lock_reason = LockReason(_lock_reason)

        def _parse_dependencies(data: object) -> None | Unset | list["Dependency"]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError
                dependencies_type_0 = []
                _dependencies_type_0 = data
                for dependencies_type_0_item_data in _dependencies_type_0:
                    dependencies_type_0_item = Dependency.from_dict(
                        dependencies_type_0_item_data
                    )

                    dependencies_type_0.append(dependencies_type_0_item)

                return dependencies_type_0
            except:  # noqa: E722
                pass
            return cast("None | Unset | list[Dependency]", data)

        dependencies = _parse_dependencies(d.pop("dependencies", UNSET))

        def _parse_type_(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        type_ = _parse_type_(d.pop("type", UNSET))

        def _parse_title(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        title = _parse_title(d.pop("title", UNSET))

        def _parse_status(data: object) -> None | Unset | int:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | int", data)

        status = _parse_status(d.pop("status", UNSET))

        def _parse_detail(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        detail = _parse_detail(d.pop("detail", UNSET))

        def _parse_instance(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        instance = _parse_instance(d.pop("instance", UNSET))

        def _parse_trace_id(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        trace_id = _parse_trace_id(d.pop("traceId", UNSET))

        data_source_object_locked_problem_details = cls(
            lock_reason=lock_reason,
            dependencies=dependencies,
            type_=type_,
            title=title,
            status=status,
            detail=detail,
            instance=instance,
            trace_id=trace_id,
        )

        return data_source_object_locked_problem_details



from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.types import UNSET, Unset

T = TypeVar("T", bound="ConflictProblemDetails")


@_attrs_define
class ConflictProblemDetails:
    """Example:
        {'title': 'Conflict', 'detail': 'The request could not be completed due to a conflict with the current state of
            the resource.', 'status': 409, 'traceId': 'd61f7ce-cccb-4e5b-8727-3b68a61a0559'}

    Attributes:
        type_ (Union[None, Unset, str]):
        title (Union[None, Unset, str]):
        status (Union[None, Unset, int]):
        detail (Union[None, Unset, str]):
        instance (Union[None, Unset, str]):
        trace_id (Union[None, Unset, str]):
    """

    type_: None | Unset | str = UNSET
    title: None | Unset | str = UNSET
    status: None | Unset | int = UNSET
    detail: None | Unset | str = UNSET
    instance: None | Unset | str = UNSET
    trace_id: None | Unset | str = UNSET

    def to_dict(self) -> dict[str, Any]:
        """Convert the object to a dictionary."""
        type_: None | Unset | str
        if isinstance(self.type_, Unset):
            type_ = UNSET
        else:
            type_ = self.type_

        title: None | Unset | str
        if isinstance(self.title, Unset):
            title = UNSET
        else:
            title = self.title

        status: None | Unset | int
        if isinstance(self.status, Unset):
            status = UNSET
        else:
            status = self.status

        detail: None | Unset | str
        if isinstance(self.detail, Unset):
            detail = UNSET
        else:
            detail = self.detail

        instance: None | Unset | str
        if isinstance(self.instance, Unset):
            instance = UNSET
        else:
            instance = self.instance

        trace_id: None | Unset | str
        if isinstance(self.trace_id, Unset):
            trace_id = UNSET
        else:
            trace_id = self.trace_id

        field_dict: dict[str, Any] = {}

        field_dict.update({})
        if type_ is not UNSET:
            field_dict["type"] = type_
        if title is not UNSET:
            field_dict["title"] = title
        if status is not UNSET:
            field_dict["status"] = status
        if detail is not UNSET:
            field_dict["detail"] = detail
        if instance is not UNSET:
            field_dict["instance"] = instance
        if trace_id is not UNSET:
            field_dict["traceId"] = trace_id

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        """Create an instance from a dictionary."""
        d = dict(src_dict)

        def _parse_type_(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        type_ = _parse_type_(d.pop("type", UNSET))

        def _parse_title(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        title = _parse_title(d.pop("title", UNSET))

        def _parse_status(data: object) -> None | Unset | int:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | int", data)

        status = _parse_status(d.pop("status", UNSET))

        def _parse_detail(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        detail = _parse_detail(d.pop("detail", UNSET))

        def _parse_instance(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        instance = _parse_instance(d.pop("instance", UNSET))

        def _parse_trace_id(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        trace_id = _parse_trace_id(d.pop("traceId", UNSET))

        conflict_problem_details = cls(
            type_=type_,
            title=title,
            status=status,
            detail=detail,
            instance=instance,
            trace_id=trace_id,
        )

        return conflict_problem_details



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from typing_extensions import Self

T = TypeVar("T", bound="Space")


@_attrs_define
class Space:
    """Example:
        {'id': 541244332157142140, 'name': 'Global Brand'}

    Attributes:
        id (int): Unique Space identifier
        name (str): Space name
    """

    id: int
    name: str

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name = self.name

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "id": id,
                "name": name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)
        id = d.pop("id")

        name = d.pop("name")

        space = cls(
            id=id,
            name=name,
        )

        return space



from pathlib import Path

import pytest

from templafy import AuthenticatedClient, Client

collect_ignore_glob = ["assets/**"]
pytest_plugins = []


@pytest.fixture(scope="session")
def assets_dir() -> Path:
    """Return a path to the test assets directory."""
    return Path(__file__).parent / "assets"


@pytest.fixture
def base_url():
    """Base URL for testing."""
    return "https://test.api.templafy.com/v3"


@pytest.fixture
def api_token():
    """Mock API token for testing."""
    return "test_token_12345"


@pytest.fixture
def mock_client(base_url):
    """Mock unauthenticated client for testing."""
    return Client(base_url=base_url)


@pytest.fixture
def mock_authenticated_client(base_url, api_token):
    """Mock authenticated client for testing."""
    return AuthenticatedClient(base_url=base_url, token=api_token)


@pytest.fixture
def mock_space_data():
    """Mock space data for testing."""
    return [
        {
            "id": "space1",
            "name": "Test Space 1",
            "description": "A test space",
            "is_active": True,
        },
        {
            "id": "space2",
            "name": "Test Space 2",
            "description": "Another test space",
            "is_active": True,
        },
    ]


@pytest.fixture
def mock_document_data():
    """Mock document data for testing."""
    return [
        {
            "id": "doc1",
            "name": "Test Document 1",
            "description": "A test document",
            "template_type": "word",
            "library_id": "lib1",
        },
        {
            "id": "doc2",
            "name": "Test Document 2",
            "description": "Another test document",
            "template_type": "powerpoint",
            "library_id": "lib1",
        },
    ]


@pytest.fixture
def mock_library_data():
    """Mock library data for testing."""
    return [
        {
            "id": "lib1",
            "name": "Test Library 1",
            "description": "A test library",
            "space_id": "space1",
            "is_active": True,
        },
    ]



from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Self

from templafy.types import UNSET, Unset

T = TypeVar("T", bound="DataSourceImageItemField")


@_attrs_define
class DataSourceImageItemField:
    """Example:
        {'dataSourceFieldId': 2, 'type': 'image', 'fileName': 'Cat', 'fileUrl':
            'https://en.wikipedia.org/wiki/Cat#/media/File:Cat_August_2010-4.jpg'}

    Attributes:
        type_ (Union[None, str]): Data source item field type.
        data_source_field_id (Union[Unset, int]): Unique data source field identifier.
        file_name (Union[None, Unset, str]): The name of the file.
        file_url (Union[None, Unset, str]): The file size must be under 2MB, and it should be in one of these formats:
            .png, .jpg, .jpeg, .gif, .bmp, .emf, .wmf, .svg.
    """

    type_: None | str
    data_source_field_id: Unset | int = UNSET
    file_name: None | Unset | str = UNSET
    file_url: None | Unset | str = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_: None | str
        type_ = self.type_

        data_source_field_id = self.data_source_field_id

        file_name: None | Unset | str
        if isinstance(self.file_name, Unset):
            file_name = UNSET
        else:
            file_name = self.file_name

        file_url: None | Unset | str
        if isinstance(self.file_url, Unset):
            file_url = UNSET
        else:
            file_url = self.file_url

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type_,
            }
        )
        if data_source_field_id is not UNSET:
            field_dict["dataSourceFieldId"] = data_source_field_id
        if file_name is not UNSET:
            field_dict["fileName"] = file_name
        if file_url is not UNSET:
            field_dict["fileUrl"] = file_url

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)

        def _parse_type_(data: object) -> None | str:
            if data is None:
                return data
            return cast("None | str", data)

        type_ = _parse_type_(d.pop("type"))

        data_source_field_id = d.pop("dataSourceFieldId", UNSET)

        def _parse_file_name(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        file_name = _parse_file_name(d.pop("fileName", UNSET))

        def _parse_file_url(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        file_url = _parse_file_url(d.pop("fileUrl", UNSET))

        data_source_image_item_field = cls(
            type_=type_,
            data_source_field_id=data_source_field_id,
            file_name=file_name,
            file_url=file_url,
        )

        data_source_image_item_field.additional_properties = d
        return data_source_image_item_field

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Self

from templafy.types import UNSET, Unset

T = TypeVar("T", bound="UpdateImageDataSourceItemFieldRequest")


@_attrs_define
class UpdateImageDataSourceItemFieldRequest:
    """Example:
        {'type': 'image', 'fileName': 'Cat', 'fileUrl':
            'https://en.wikipedia.org/wiki/Cat#/media/File:Cat_August_2010-4.jpg'}

    Attributes:
        type_ (str): Data source item field type.
        file_name (str): The name of the file.
        file_url (Union[None, Unset, str]): The file size must be under 2MB, and it should be in one of these formats:
            .png, .jpg, .jpeg, .gif, .bmp, .emf, .wmf, .svg.
        content (Union[None, Unset, str]): The base64 content size must be under 2MB, and it should be in one of these
            formats: .png, .jpg, .jpeg, .gif, .bmp, .emf, .wmf, .svg.
    """

    type_: str
    file_name: str
    file_url: None | Unset | str = UNSET
    content: None | Unset | str = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        file_name = self.file_name

        file_url: None | Unset | str
        if isinstance(self.file_url, Unset):
            file_url = UNSET
        else:
            file_url = self.file_url

        content: None | Unset | str
        if isinstance(self.content, Unset):
            content = UNSET
        else:
            content = self.content

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type_,
                "fileName": file_name,
            }
        )
        if file_url is not UNSET:
            field_dict["fileUrl"] = file_url
        if content is not UNSET:
            field_dict["content"] = content

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)
        type_ = d.pop("type")

        file_name = d.pop("fileName")

        def _parse_file_url(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        file_url = _parse_file_url(d.pop("fileUrl", UNSET))

        def _parse_content(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        content = _parse_content(d.pop("content", UNSET))

        update_image_data_source_item_field_request = cls(
            type_=type_,
            file_name=file_name,
            file_url=file_url,
            content=content,
        )

        update_image_data_source_item_field_request.additional_properties = d
        return update_image_data_source_item_field_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class LibraryType(str, Enum):
    """Enumeration of library categories used by the API.

    Each value identifies a specific type of library containing assets such
    as documents, images, slides, or other element collections.
    """

    DOCUMENTS = "documents"
    EMAIL_ELEMENTS = "email-elements"
    IMAGES = "images"
    LINKS = "links"
    PDFS = "pdfs"
    PRESENTATIONS = "presentations"
    SLIDES = "slides"
    SLIDE_ELEMENTS = "slide-elements"
    SPREADSHEETS = "spreadsheets"
    TEXT_ELEMENTS = "text-elements"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Self

from templafy.types import UNSET, Unset

T = TypeVar("T", bound="PatchTextDataSourceItemFieldRequest")


@_attrs_define
class PatchTextDataSourceItemFieldRequest:
    """Example:
        {'dataSourceFieldId': 0, 'type': 'text', 'value': 'An updated value'}

    Attributes:
        type_ (str): Data source item field type.
        data_source_field_id (int): The identifier of the field to be updated.
        value (Union[None, Unset, str]): Text data source item field value. Max length is 8000 characters.
    """

    type_: str
    data_source_field_id: int
    value: None | Unset | str = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        data_source_field_id = self.data_source_field_id

        value: None | Unset | str
        if isinstance(self.value, Unset):
            value = UNSET
        else:
            value = self.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type_,
                "dataSourceFieldId": data_source_field_id,
            }
        )
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)
        type_ = d.pop("type")

        data_source_field_id = d.pop("dataSourceFieldId")

        def _parse_value(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        value = _parse_value(d.pop("value", UNSET))

        patch_text_data_source_item_field_request = cls(
            type_=type_,
            data_source_field_id=data_source_field_id,
            value=value,
        )

        patch_text_data_source_item_field_request.additional_properties = d
        return patch_text_data_source_item_field_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from io import BytesIO
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Self

from templafy import types
from templafy.types import UNSET, File, Unset

T = TypeVar("T", bound="PostLibrariesSpaceIdPdfsFoldersFolderIdAssetsBody")


@_attrs_define
class PostLibrariesSpaceIdPdfsFoldersFolderIdAssetsBody:
    """Attributes:
    file (File): A file to be uploaded. The maximum file size is 50 mb
    name (Union[Unset, str]): The name is inferred from the file name by default. It can be overridden by providing
        a different value with this field
    description (Union[Unset, str]): Description is used to specify the intended usage of the asset
    tags (Union[Unset, list[str]]): Tags should describe the content of the asset making it easier for a user to
        locate it
    external_data (Union[Unset, str]): External data which can be attached for future reference
    """

    file: File
    name: Unset | str = UNSET
    description: Unset | str = UNSET
    tags: Unset | list[str] = UNSET
    external_data: Unset | str = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        file = self.file.to_tuple()

        name = self.name

        description = self.description

        tags: Unset | list[str] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        external_data = self.external_data

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "File": file,
            }
        )
        if name is not UNSET:
            field_dict["Name"] = name
        if description is not UNSET:
            field_dict["Description"] = description
        if tags is not UNSET:
            field_dict["Tags"] = tags
        if external_data is not UNSET:
            field_dict["ExternalData"] = external_data

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("File", self.file.to_tuple()))

        if not isinstance(self.name, Unset):
            files.append(("Name", (None, str(self.name).encode(), "text/plain")))

        if not isinstance(self.description, Unset):
            files.append(
                ("Description", (None, str(self.description).encode(), "text/plain"))
            )

        if not isinstance(self.tags, Unset):
            for tags_item_element in self.tags:
                files.append(
                    ("Tags", (None, str(tags_item_element).encode(), "text/plain"))
                )

        if not isinstance(self.external_data, Unset):
            files.append(
                ("ExternalData", (None, str(self.external_data).encode(), "text/plain"))
            )

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)
        file = File(payload=BytesIO(d.pop("File")))

        name = d.pop("Name", UNSET)

        description = d.pop("Description", UNSET)

        tags = cast("list[str]", d.pop("Tags", UNSET))

        external_data = d.pop("ExternalData", UNSET)

        post_libraries_space_id_pdfs_folders_folder_id_assets_body = cls(
            file=file,
            name=name,
            description=description,
            tags=tags,
            external_data=external_data,
        )

        post_libraries_space_id_pdfs_folders_folder_id_assets_body.additional_properties = d
        return post_libraries_space_id_pdfs_folders_folder_id_assets_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class AssetStateWithoutFile(str, Enum):
    """Enumeration of possible states for an asset that has no file
    attached.

    Currently only indicates readiness state returned by the API.
    """

    READY = "ready"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.models.asset_file_state_with_previews import AssetFileStateWithPreviews
from templafy.types import UNSET, Unset

if TYPE_CHECKING:
    from templafy.models.dimensions import Dimensions
from templafy.models.dimensions import Dimensions

T = TypeVar("T", bound="PresentationBase")


@_attrs_define
class PresentationBase:
    """Attributes:
    asset_type (str):
    asset_state (AssetFileStateWithPreviews): The current state of the asset
    dimensions (Dimensions): Asset dimensions
    id (int): Unique asset identifier
    folder_id (int): Unique folder identifier
    name (str): Display name
    tags (list[str]):
    file_size (int): File size in bytes
    checksum (str): MD5 checksum of the bytes
    file_extension (str): Suffix to the name of the file
    navigation_path (str): Hierarchical path in lowercase based on the location of a presentation. E.g.
        "folder-a/folder-b/_my-presentation" when the location is "Folder A > Folder B > My Presentation"
    modified_at (str): Date and time in ISO 8601 format of when the asset was last modified
    external_data (Union[None, Unset, str]): External data which can be attached for future reference
    small_preview_link (Union[None, Unset, str]): Link to the asset with the maximum width 400px
    large_preview_link (Union[None, Unset, str]): Link to the asset with the maximum width 1500px
    """

    asset_type: str
    asset_state: AssetFileStateWithPreviews
    dimensions: "Dimensions"
    id: int
    folder_id: int
    name: str
    tags: list[str]
    file_size: int
    checksum: str
    file_extension: str
    navigation_path: str
    modified_at: str
    external_data: None | Unset | str = UNSET
    small_preview_link: None | Unset | str = UNSET
    large_preview_link: None | Unset | str = UNSET

    def to_dict(self) -> dict[str, Any]:
        asset_type = self.asset_type

        asset_state = self.asset_state.value

        dimensions = self.dimensions.to_dict()

        id = self.id

        folder_id = self.folder_id

        name = self.name

        tags = self.tags

        file_size = self.file_size

        checksum = self.checksum

        file_extension = self.file_extension

        navigation_path = self.navigation_path

        modified_at = self.modified_at

        external_data: None | Unset | str
        if isinstance(self.external_data, Unset):
            external_data = UNSET
        else:
            external_data = self.external_data

        small_preview_link: None | Unset | str
        if isinstance(self.small_preview_link, Unset):
            small_preview_link = UNSET
        else:
            small_preview_link = self.small_preview_link

        large_preview_link: None | Unset | str
        if isinstance(self.large_preview_link, Unset):
            large_preview_link = UNSET
        else:
            large_preview_link = self.large_preview_link

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "assetType": asset_type,
                "assetState": asset_state,
                "dimensions": dimensions,
                "id": id,
                "folderId": folder_id,
                "name": name,
                "tags": tags,
                "fileSize": file_size,
                "checksum": checksum,
                "fileExtension": file_extension,
                "navigationPath": navigation_path,
                "modifiedAt": modified_at,
            }
        )
        if external_data is not UNSET:
            field_dict["externalData"] = external_data
        if small_preview_link is not UNSET:
            field_dict["smallPreviewLink"] = small_preview_link
        if large_preview_link is not UNSET:
            field_dict["largePreviewLink"] = large_preview_link

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        # Top-level imports are used to avoid inline imports (PLC0415)
        d = dict(src_dict)
        asset_type = d.pop("assetType")

        asset_state = AssetFileStateWithPreviews(d.pop("assetState"))

        dimensions = Dimensions.from_dict(d.pop("dimensions"))

        id = d.pop("id")

        folder_id = d.pop("folderId")

        name = d.pop("name")

        tags = cast("list[str]", d.pop("tags"))

        file_size = d.pop("fileSize")

        checksum = d.pop("checksum")

        file_extension = d.pop("fileExtension")

        navigation_path = d.pop("navigationPath")

        modified_at = d.pop("modifiedAt")

        def _parse_external_data(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        external_data = _parse_external_data(d.pop("externalData", UNSET))

        def _parse_small_preview_link(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        small_preview_link = _parse_small_preview_link(d.pop("smallPreviewLink", UNSET))

        def _parse_large_preview_link(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        large_preview_link = _parse_large_preview_link(d.pop("largePreviewLink", UNSET))

        presentation_base = cls(
            asset_type=asset_type,
            asset_state=asset_state,
            dimensions=dimensions,
            id=id,
            folder_id=folder_id,
            name=name,
            tags=tags,
            file_size=file_size,
            checksum=checksum,
            file_extension=file_extension,
            navigation_path=navigation_path,
            modified_at=modified_at,
            external_data=external_data,
            small_preview_link=small_preview_link,
            large_preview_link=large_preview_link,
        )

        return presentation_base



from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.models.source_entity_type import SourceEntityType
from templafy.types import UNSET, Unset

T = TypeVar("T", bound="Dependency")


@_attrs_define
class Dependency:
    """The model describing a dependency.

    Attributes:
        source_entity_type (Union[Unset, SourceEntityType]):
        source_entity_id (Union[None, Unset, str]): The id of the dependency source.
        description (Union[None, Unset, str]): Human readable description of the source type.
    """

    source_entity_type: Unset | SourceEntityType = UNSET
    source_entity_id: None | Unset | str = UNSET
    description: None | Unset | str = UNSET

    def to_dict(self) -> dict[str, Any]:
        source_entity_type: Unset | str = UNSET
        if not isinstance(self.source_entity_type, Unset):
            source_entity_type = self.source_entity_type.value

        source_entity_id: None | Unset | str
        if isinstance(self.source_entity_id, Unset):
            source_entity_id = UNSET
        else:
            source_entity_id = self.source_entity_id

        description: None | Unset | str
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        field_dict: dict[str, Any] = {}

        field_dict.update({})
        if source_entity_type is not UNSET:
            field_dict["sourceEntityType"] = source_entity_type
        if source_entity_id is not UNSET:
            field_dict["sourceEntityId"] = source_entity_id
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)
        _source_entity_type = d.pop("sourceEntityType", UNSET)
        source_entity_type: Unset | SourceEntityType
        if isinstance(_source_entity_type, Unset):
            source_entity_type = UNSET
        else:
            source_entity_type = SourceEntityType(_source_entity_type)

        def _parse_source_entity_id(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        source_entity_id = _parse_source_entity_id(d.pop("sourceEntityId", UNSET))

        def _parse_description(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        description = _parse_description(d.pop("description", UNSET))

        dependency = cls(
            source_entity_type=source_entity_type,
            source_entity_id=source_entity_id,
            description=description,
        )

        return dependency



from enum import Enum


class FolderState(str, Enum):
    """Enumeration of possible lifecycle states for a folder resource.

    Typical values indicate whether the folder is ready for use or in the
    process of being deleted on the server.
    """

    DELETING = "deleting"
    READY = "ready"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class AssetFileStateWithoutPreviews(str, Enum):
    """Enumeration of possible file processing states for an asset that
    does not include generated previews.

    Values indicate server-side lifecycle stages for an asset's file only.
    """

    DELETING = "deleting"
    PROCESSING = "processing"
    PROCESSINGFAILED = "processingFailed"
    READY = "ready"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from io import BytesIO
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Self

from templafy import types
from templafy.types import UNSET, File, FileTypes, Unset

T = TypeVar("T", bound="PatchLibrariesSpaceIdPdfsAssetsAssetIdBody")


@_attrs_define
class PatchLibrariesSpaceIdPdfsAssetsAssetIdBody:
    """Attributes:
    folder_id (Union[Unset, int]): The identifier of the destination folder
    name (Union[Unset, str]): A new display name of the asset
    description (Union[Unset, str]): Description is used to specify the intended usage of the asset
    tags (Union[Unset, list[str]]): Tags should describe the content of the asset making it easier for a user to
        locate it
    external_data (Union[Unset, str]): External data which can be attached for future reference
    file (Union[Unset, File]): A file to be uploaded. The maximum file size is 50 mb
    """

    folder_id: Unset | int = UNSET
    name: Unset | str = UNSET
    description: Unset | str = UNSET
    tags: Unset | list[str] = UNSET
    external_data: Unset | str = UNSET
    file: Unset | File = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        folder_id = self.folder_id

        name = self.name

        description = self.description

        tags: Unset | list[str] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        external_data = self.external_data

        file: Unset | FileTypes = UNSET
        if not isinstance(self.file, Unset):
            file = self.file.to_tuple()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if folder_id is not UNSET:
            field_dict["FolderId"] = folder_id
        if name is not UNSET:
            field_dict["Name"] = name
        if description is not UNSET:
            field_dict["Description"] = description
        if tags is not UNSET:
            field_dict["Tags"] = tags
        if external_data is not UNSET:
            field_dict["ExternalData"] = external_data
        if file is not UNSET:
            field_dict["File"] = file

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.folder_id, Unset):
            files.append(
                ("FolderId", (None, str(self.folder_id).encode(), "text/plain"))
            )

        if not isinstance(self.name, Unset):
            files.append(("Name", (None, str(self.name).encode(), "text/plain")))

        if not isinstance(self.description, Unset):
            files.append(
                ("Description", (None, str(self.description).encode(), "text/plain"))
            )

        if not isinstance(self.tags, Unset):
            for tags_item_element in self.tags:
                files.append(
                    ("Tags", (None, str(tags_item_element).encode(), "text/plain"))
                )

        if not isinstance(self.external_data, Unset):
            files.append(
                ("ExternalData", (None, str(self.external_data).encode(), "text/plain"))
            )

        if not isinstance(self.file, Unset):
            files.append(("File", self.file.to_tuple()))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)
        folder_id = d.pop("FolderId", UNSET)

        name = d.pop("Name", UNSET)

        description = d.pop("Description", UNSET)

        tags = cast("list[str]", d.pop("Tags", UNSET))

        external_data = d.pop("ExternalData", UNSET)

        _file = d.pop("File", UNSET)
        file: Unset | File
        if isinstance(_file, Unset):
            file = UNSET
        else:
            file = File(payload=BytesIO(_file))

        patch_libraries_space_id_pdfs_assets_asset_id_body = cls(
            folder_id=folder_id,
            name=name,
            description=description,
            tags=tags,
            external_data=external_data,
            file=file,
        )

        patch_libraries_space_id_pdfs_assets_asset_id_body.additional_properties = d
        return patch_libraries_space_id_pdfs_assets_asset_id_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, cast

import httpx

from templafy import errors
from templafy.client import AuthenticatedClient, Client
from templafy.models.email_element import EmailElement
from templafy.models.not_found_problem_details import NotFoundProblemDetails
from templafy.models.validation_problem_details import ValidationProblemDetails
from templafy.types import UNSET, Response, Unset


def _get_kwargs(
    space_id: int,
    folder_id: int,
    *,
    search_query: Unset | str = UNSET,
    page_number: Unset | int = UNSET,
    page_size: Unset | int = UNSET,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    params["searchQuery"] = search_query

    params["pageNumber"] = page_number

    params["pageSize"] = page_size

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": f"/libraries/{space_id}/email-elements/folders/{folder_id}/assets",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> (
    Any
    | NotFoundProblemDetails
    | ValidationProblemDetails
    | list["EmailElement"]
    | None
):
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = EmailElement.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200

    if response.status_code == 400:
        response_400 = ValidationProblemDetails.from_dict(response.json())

        return response_400

    if response.status_code == 401:
        response_401 = cast("Any", None)
        return response_401

    if response.status_code == 403:
        response_403 = cast("Any", None)
        return response_403

    if response.status_code == 404:
        response_404 = NotFoundProblemDetails.from_dict(response.json())

        return response_404

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[
    Any | NotFoundProblemDetails | ValidationProblemDetails | list["EmailElement"]
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    space_id: int,
    folder_id: int,
    *,
    client: AuthenticatedClient,
    search_query: Unset | str = UNSET,
    page_number: Unset | int = UNSET,
    page_size: Unset | int = UNSET,
) -> Response[
    Any | NotFoundProblemDetails | ValidationProblemDetails | list["EmailElement"]
]:
    """Lists all email elements in the folder.

     The result does not include email elements from subfolders.When {searchQuery} is used the result
    includes email elements from subfolders. The search mode enables pagination with 1000 page size
    maximum.

    Args:
        space_id (int):
        folder_id (int):
        search_query (Union[Unset, str]):
        page_number (Union[Unset, int]):
        page_size (Union[Unset, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, NotFoundProblemDetails, ValidationProblemDetails, list['EmailElement']]]
    """

    kwargs = _get_kwargs(
        space_id=space_id,
        folder_id=folder_id,
        search_query=search_query,
        page_number=page_number,
        page_size=page_size,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    space_id: int,
    folder_id: int,
    *,
    client: AuthenticatedClient,
    search_query: Unset | str = UNSET,
    page_number: Unset | int = UNSET,
    page_size: Unset | int = UNSET,
) -> (
    Any
    | NotFoundProblemDetails
    | ValidationProblemDetails
    | list["EmailElement"]
    | None
):
    """Lists all email elements in the folder.

     The result does not include email elements from subfolders.When {searchQuery} is used the result
    includes email elements from subfolders. The search mode enables pagination with 1000 page size
    maximum.

    Args:
        space_id (int):
        folder_id (int):
        search_query (Union[Unset, str]):
        page_number (Union[Unset, int]):
        page_size (Union[Unset, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, NotFoundProblemDetails, ValidationProblemDetails, list['EmailElement']]
    """

    return sync_detailed(
        space_id=space_id,
        folder_id=folder_id,
        client=client,
        search_query=search_query,
        page_number=page_number,
        page_size=page_size,
    ).parsed


async def asyncio_detailed(
    space_id: int,
    folder_id: int,
    *,
    client: AuthenticatedClient,
    search_query: Unset | str = UNSET,
    page_number: Unset | int = UNSET,
    page_size: Unset | int = UNSET,
) -> Response[
    Any | NotFoundProblemDetails | ValidationProblemDetails | list["EmailElement"]
]:
    """Lists all email elements in the folder.

     The result does not include email elements from subfolders.When {searchQuery} is used the result
    includes email elements from subfolders. The search mode enables pagination with 1000 page size
    maximum.

    Args:
        space_id (int):
        folder_id (int):
        search_query (Union[Unset, str]):
        page_number (Union[Unset, int]):
        page_size (Union[Unset, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, NotFoundProblemDetails, ValidationProblemDetails, list['EmailElement']]]
    """

    kwargs = _get_kwargs(
        space_id=space_id,
        folder_id=folder_id,
        search_query=search_query,
        page_number=page_number,
        page_size=page_size,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    space_id: int,
    folder_id: int,
    *,
    client: AuthenticatedClient,
    search_query: Unset | str = UNSET,
    page_number: Unset | int = UNSET,
    page_size: Unset | int = UNSET,
) -> (
    Any
    | NotFoundProblemDetails
    | ValidationProblemDetails
    | list["EmailElement"]
    | None
):
    """Lists all email elements in the folder.

     The result does not include email elements from subfolders.When {searchQuery} is used the result
    includes email elements from subfolders. The search mode enables pagination with 1000 page size
    maximum.

    Args:
        space_id (int):
        folder_id (int):
        search_query (Union[Unset, str]):
        page_number (Union[Unset, int]):
        page_size (Union[Unset, int]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, NotFoundProblemDetails, ValidationProblemDetails, list['EmailElement']]
    """

    return (
        await asyncio_detailed(
            space_id=space_id,
            folder_id=folder_id,
            client=client,
            search_query=search_query,
            page_number=page_number,
            page_size=page_size,
        )
    ).parsed



from enum import Enum


class SourceEntityType(str, Enum):
    """Enumeration of source entity types that can be referenced by the API.

    Identifies whether the source is a data source, a data source item, or
    some other external entity.
    """

    DATASOURCE = "dataSource"
    DATASOURCEITEM = "dataSourceItem"
    OTHER = "other"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Self

from templafy.types import UNSET, Unset

T = TypeVar("T", bound="CreateTextFieldSchemaRequest")


@_attrs_define
class CreateTextFieldSchemaRequest:
    """Example:
        {'name': 'History', 'type': 'text', 'isMultipleLines': True, 'defaultValue': 'The city was established in the
            year 1652 by Dutch explorers...', 'isRequired': False}

    Attributes:
        type_ (str): Data source field schema type.
        name (str): The name of the field. It must be unique within the data source.
        is_required (Union[Unset, bool]): Whether the field is required. If true, the field must be filled in when
            creating a data source item.
        is_multiple_lines (Union[Unset, bool]): Whether the field is multiple lines. If true, the field will be rendered
            as a text area.
        default_value (Union[None, Unset, str]): The default value of the field. If specified, the field will be pre-
            filled with this value when creating a data source item.
    """

    type_: str
    name: str
    is_required: Unset | bool = UNSET
    is_multiple_lines: Unset | bool = UNSET
    default_value: None | Unset | str = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        name = self.name

        is_required = self.is_required

        is_multiple_lines = self.is_multiple_lines

        default_value: None | Unset | str
        if isinstance(self.default_value, Unset):
            default_value = UNSET
        else:
            default_value = self.default_value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type_,
                "name": name,
            }
        )
        if is_required is not UNSET:
            field_dict["isRequired"] = is_required
        if is_multiple_lines is not UNSET:
            field_dict["isMultipleLines"] = is_multiple_lines
        if default_value is not UNSET:
            field_dict["defaultValue"] = default_value

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)
        type_ = d.pop("type")

        name = d.pop("name")

        is_required = d.pop("isRequired", UNSET)

        is_multiple_lines = d.pop("isMultipleLines", UNSET)

        def _parse_default_value(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        default_value = _parse_default_value(d.pop("defaultValue", UNSET))

        create_text_field_schema_request = cls(
            type_=type_,
            name=name,
            is_required=is_required,
            is_multiple_lines=is_multiple_lines,
            default_value=default_value,
        )

        create_text_field_schema_request.additional_properties = d
        return create_text_field_schema_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Union, cast

import httpx

from templafy import errors
from templafy.client import AuthenticatedClient, Client
from templafy.models.color_theme_field_schema import ColorThemeFieldSchema
from templafy.models.create_image_field_schema_request import (
    CreateImageFieldSchemaRequest,
)
from templafy.models.create_number_field_schema_request import (
    CreateNumberFieldSchemaRequest,
)
from templafy.models.create_reference_field_schema_request import (
    CreateReferenceFieldSchemaRequest,
)
from templafy.models.create_text_field_schema_request import (
    CreateTextFieldSchemaRequest,
)
from templafy.models.font_field_schema import FontFieldSchema
from templafy.models.image_field_schema import ImageFieldSchema
from templafy.models.language_field_schema import LanguageFieldSchema
from templafy.models.not_found_problem_details import NotFoundProblemDetails
from templafy.models.number_field_schema import NumberFieldSchema
from templafy.models.reference_field_schema import ReferenceFieldSchema
from templafy.models.text_field_schema import TextFieldSchema
from templafy.models.validation_problem_details import ValidationProblemDetails
from templafy.types import Response


def _get_kwargs(
    data_source_id: int,
    *,
    body: Union[
        "CreateImageFieldSchemaRequest",
        "CreateNumberFieldSchemaRequest",
        "CreateReferenceFieldSchemaRequest",
        "CreateTextFieldSchemaRequest",
    ],
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": f"/data-sources/{data_source_id}/fields",
    }

    if isinstance(
        body,
        Union[
            "CreateImageFieldSchemaRequest",
            "CreateNumberFieldSchemaRequest",
            "CreateReferenceFieldSchemaRequest",
            "CreateTextFieldSchemaRequest",
        ],
    ):
        _kwargs["json"] = body.to_dict()

        headers["Content-Type"] = "application/json-patch+json"
    if isinstance(
        body,
        Union[
            "CreateImageFieldSchemaRequest",
            "CreateNumberFieldSchemaRequest",
            "CreateReferenceFieldSchemaRequest",
            "CreateTextFieldSchemaRequest",
        ],
    ):
        _kwargs["json"] = body.to_dict()

        headers["Content-Type"] = "application/json"
    if isinstance(
        body,
        Union[
            "CreateImageFieldSchemaRequest",
            "CreateNumberFieldSchemaRequest",
            "CreateReferenceFieldSchemaRequest",
            "CreateTextFieldSchemaRequest",
        ],
    ):
        _kwargs["json"] = body.to_dict()

        headers["Content-Type"] = "application/*+json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> (
    Any
    | NotFoundProblemDetails
    | Union[
        "ColorThemeFieldSchema",
        "FontFieldSchema",
        "ImageFieldSchema",
        "LanguageFieldSchema",
        "NumberFieldSchema",
        "ReferenceFieldSchema",
        "TextFieldSchema",
    ]
    | ValidationProblemDetails
    | None
):
    if response.status_code == 201:

        def _parse_response_201(
            data: object,
        ) -> Union[
            "ColorThemeFieldSchema",
            "FontFieldSchema",
            "ImageFieldSchema",
            "LanguageFieldSchema",
            "NumberFieldSchema",
            "ReferenceFieldSchema",
            "TextFieldSchema",
        ]:
            try:
                if not isinstance(data, dict):
                    raise TypeError
                response_201_type_0 = TextFieldSchema.from_dict(data)

                return response_201_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError
                response_201_type_1 = NumberFieldSchema.from_dict(data)

                return response_201_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError
                response_201_type_2 = ReferenceFieldSchema.from_dict(data)

                return response_201_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError
                response_201_type_3 = ImageFieldSchema.from_dict(data)

                return response_201_type_3
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError
                response_201_type_4 = LanguageFieldSchema.from_dict(data)

                return response_201_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError
                response_201_type_5 = FontFieldSchema.from_dict(data)

                return response_201_type_5
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError
            response_201_type_6 = ColorThemeFieldSchema.from_dict(data)

            return response_201_type_6

        response_201 = _parse_response_201(response.json())

        return response_201

    if response.status_code == 400:
        response_400 = ValidationProblemDetails.from_dict(response.json())

        return response_400

    if response.status_code == 401:
        response_401 = cast("Any", None)
        return response_401

    if response.status_code == 403:
        response_403 = cast("Any", None)
        return response_403

    if response.status_code == 404:
        response_404 = NotFoundProblemDetails.from_dict(response.json())

        return response_404

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[
    Any
    | NotFoundProblemDetails
    | Union[
        "ColorThemeFieldSchema",
        "FontFieldSchema",
        "ImageFieldSchema",
        "LanguageFieldSchema",
        "NumberFieldSchema",
        "ReferenceFieldSchema",
        "TextFieldSchema",
    ]
    | ValidationProblemDetails
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    data_source_id: int,
    *,
    client: AuthenticatedClient,
    body: Union[
        "CreateImageFieldSchemaRequest",
        "CreateNumberFieldSchemaRequest",
        "CreateReferenceFieldSchemaRequest",
        "CreateTextFieldSchemaRequest",
    ],
) -> Response[
    Any
    | NotFoundProblemDetails
    | Union[
        "ColorThemeFieldSchema",
        "FontFieldSchema",
        "ImageFieldSchema",
        "LanguageFieldSchema",
        "NumberFieldSchema",
        "ReferenceFieldSchema",
        "TextFieldSchema",
    ]
    | ValidationProblemDetails
]:
    """Creates a new data source field.

     Extends a data source by adding a new field.

    Args:
        data_source_id (int): The ID of the data source.
        client (AuthenticatedClient): The authenticated client instance.
        body (Union['CreateImageFieldSchemaRequest', 'CreateNumberFieldSchemaRequest',
            'CreateReferenceFieldSchemaRequest', 'CreateTextFieldSchemaRequest']):  Example: {'name':
            'History', 'type': 'text', 'isMultipleLines': True, 'defaultValue': 'The city was
            established in the year 1652 by Dutch explorers...', 'isRequired': False}.

    Note:
            body may be one of several concrete create request types depending on the field
            schema being created.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, NotFoundProblemDetails, Union['ColorThemeFieldSchema', 'FontFieldSchema', 'ImageFieldSchema', 'LanguageFieldSchema', 'NumberFieldSchema', 'ReferenceFieldSchema', 'TextFieldSchema'], ValidationProblemDetails]]
    """

    kwargs = _get_kwargs(
        data_source_id=data_source_id,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    data_source_id: int,
    *,
    client: AuthenticatedClient,
    body: Union[
        "CreateImageFieldSchemaRequest",
        "CreateNumberFieldSchemaRequest",
        "CreateReferenceFieldSchemaRequest",
        "CreateTextFieldSchemaRequest",
    ],
) -> (
    Any
    | NotFoundProblemDetails
    | Union[
        "ColorThemeFieldSchema",
        "FontFieldSchema",
        "ImageFieldSchema",
        "LanguageFieldSchema",
        "NumberFieldSchema",
        "ReferenceFieldSchema",
        "TextFieldSchema",
    ]
    | ValidationProblemDetails
    | None
):
    """Creates a new data source field.

     Extends a data source by adding a new field.

    Args:
        data_source_id (int): The ID of the data source.
        client (AuthenticatedClient): The authenticated client instance.
        body (Union['CreateImageFieldSchemaRequest', 'CreateNumberFieldSchemaRequest',
            'CreateReferenceFieldSchemaRequest', 'CreateTextFieldSchemaRequest']):  Example: {'name':
            'History', 'type': 'text', 'isMultipleLines': True, 'defaultValue': 'The city was
            established in the year 1652 by Dutch explorers...', 'isRequired': False}.

    Note:
            body may be one of several concrete create request types depending on the field
            schema being created.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, NotFoundProblemDetails, Union['ColorThemeFieldSchema', 'FontFieldSchema', 'ImageFieldSchema', 'LanguageFieldSchema', 'NumberFieldSchema', 'ReferenceFieldSchema', 'TextFieldSchema'], ValidationProblemDetails]
    """

    return sync_detailed(
        data_source_id=data_source_id,
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    data_source_id: int,
    *,
    client: AuthenticatedClient,
    body: Union[
        "CreateImageFieldSchemaRequest",
        "CreateNumberFieldSchemaRequest",
        "CreateReferenceFieldSchemaRequest",
        "CreateTextFieldSchemaRequest",
    ],
) -> Response[
    Any
    | NotFoundProblemDetails
    | Union[
        "ColorThemeFieldSchema",
        "FontFieldSchema",
        "ImageFieldSchema",
        "LanguageFieldSchema",
        "NumberFieldSchema",
        "ReferenceFieldSchema",
        "TextFieldSchema",
    ]
    | ValidationProblemDetails
]:
    """Creates a new data source field.

     Extends a data source by adding a new field.

    Args:
        data_source_id (int): The ID of the data source.
        client (AuthenticatedClient): The authenticated client instance.
        body (Union['CreateImageFieldSchemaRequest', 'CreateNumberFieldSchemaRequest',
            'CreateReferenceFieldSchemaRequest', 'CreateTextFieldSchemaRequest']):  Example: {'name':
            'History', 'type': 'text', 'isMultipleLines': True, 'defaultValue': 'The city was
            established in the year 1652 by Dutch explorers...', 'isRequired': False}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, NotFoundProblemDetails, Union['ColorThemeFieldSchema', 'FontFieldSchema', 'ImageFieldSchema', 'LanguageFieldSchema', 'NumberFieldSchema', 'ReferenceFieldSchema', 'TextFieldSchema'], ValidationProblemDetails]]
    """

    kwargs = _get_kwargs(
        data_source_id=data_source_id,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    data_source_id: int,
    *,
    client: AuthenticatedClient,
    body: Union[
        "CreateImageFieldSchemaRequest",
        "CreateNumberFieldSchemaRequest",
        "CreateReferenceFieldSchemaRequest",
        "CreateTextFieldSchemaRequest",
    ],
) -> (
    Any
    | NotFoundProblemDetails
    | Union[
        "ColorThemeFieldSchema",
        "FontFieldSchema",
        "ImageFieldSchema",
        "LanguageFieldSchema",
        "NumberFieldSchema",
        "ReferenceFieldSchema",
        "TextFieldSchema",
    ]
    | ValidationProblemDetails
    | None
):
    """Creates a new data source field.

     Extends a data source by adding a new field.

    Args:
        data_source_id (int): The ID of the data source.
        client (AuthenticatedClient): The authenticated client instance.
        body (Union['CreateImageFieldSchemaRequest', 'CreateNumberFieldSchemaRequest',
            'CreateReferenceFieldSchemaRequest', 'CreateTextFieldSchemaRequest']):  Example: {'name':
            'History', 'type': 'text', 'isMultipleLines': True, 'defaultValue': 'The city was
            established in the year 1652 by Dutch explorers...', 'isRequired': False}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, NotFoundProblemDetails, Union['ColorThemeFieldSchema', 'FontFieldSchema', 'ImageFieldSchema', 'LanguageFieldSchema', 'NumberFieldSchema', 'ReferenceFieldSchema', 'TextFieldSchema'], ValidationProblemDetails]
    """

    return (
        await asyncio_detailed(
            data_source_id=data_source_id,
            client=client,
            body=body,
        )
    ).parsed



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from typing_extensions import Self

T = TypeVar("T", bound="CreateFolderRequest")


@_attrs_define
class CreateFolderRequest:
    """The request model to create a folder

    Attributes:
        name (str): Display name
    """

    name: str

    def to_dict(self) -> dict[str, Any]:
        """Convert the object to a dictionary."""
        name = self.name

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "name": name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        """Create an instance from a dictionary."""
        d = dict(src_dict)
        name = d.pop("name")

        create_folder_request = cls(
            name=name,
        )

        return create_folder_request



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.types import UNSET, Unset

if TYPE_CHECKING:
    from templafy.models.data_source_color_theme_item_field import (
        DataSourceColorThemeItemField,
    )
    from templafy.models.data_source_font_item_field import DataSourceFontItemField
    from templafy.models.data_source_image_item_field import DataSourceImageItemField
    from templafy.models.data_source_language_item_field import (
        DataSourceLanguageItemField,
    )
    from templafy.models.data_source_number_item_field import DataSourceNumberItemField
    from templafy.models.data_source_reference_item_field import (
        DataSourceReferenceItemField,
    )
    from templafy.models.data_source_text_item_field import DataSourceTextItemField

# Top-level runtime imports to satisfy PLC0415
from templafy.models.data_source_color_theme_item_field import (
    DataSourceColorThemeItemField,
)
from templafy.models.data_source_font_item_field import DataSourceFontItemField
from templafy.models.data_source_image_item_field import DataSourceImageItemField
from templafy.models.data_source_language_item_field import (
    DataSourceLanguageItemField,
)
from templafy.models.data_source_number_item_field import DataSourceNumberItemField
from templafy.models.data_source_reference_item_field import (
    DataSourceReferenceItemField,
)
from templafy.models.data_source_text_item_field import DataSourceTextItemField

T = TypeVar("T", bound="DataSourceItem")


@_attrs_define
class DataSourceItem:
    """Example:
        {'id': 638247997470215700, 'fields': [{'type': 'text', 'dataSourceFieldId': 0, 'value': 'Sample text'}, {'type':
            'number', 'dataSourceFieldId': 1, 'value': 123.4}, {'type': 'reference', 'dataSourceFieldId': 2, 'value':
            '638248473165588903'}, {'type': 'image', 'dataSourceFieldId': 3, 'fileName': 'Cat', 'fileUrl':
            'https://en.wikipedia.org/wiki/Cat#/media/File:Cat_August_2010-4.jpg'}, {'type': 'font', 'dataSourceFieldId': 4,
            'fileName': 'best-font', 'fileUrl': 'https://allfonts.com/best-font'}, {'type': 'colorTheme',
            'dataSourceFieldId': 5, 'xmlValue': '<a:clrScheme
            xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main" name="Templafy_New"><a:dk1><a:srgbClr
            val="0078FF"/></a:dk1><a:lt1><a:sysClr val="window" lastClr="FFFFFF"/></a:lt1><a:dk2><a:srgbClr
            val="000000"/></a:dk2><a:lt2><a:srgbClr val="E5E5E5"/></a:lt2><a:accent1><a:srgbClr
            val="0078FF"/></a:accent1><a:accent2><a:srgbClr val="575757"/></a:accent2><a:accent3><a:srgbClr
            val="12AA96"/></a:accent3><a:accent4><a:srgbClr val="15436B"/></a:accent4><a:accent5><a:srgbClr
            val="D44849"/></a:accent5><a:accent6><a:srgbClr val="7F7F7F"/></a:accent6><a:hlink><a:srgbClr
            val="0078FF"/></a:hlink><a:folHlink><a:srgbClr val="55CBFF"/></a:folHlink></a:clrScheme>'}, {'type': 'language',
            'dataSourceFieldId': 6, 'cultureName': 'en-GB'}]}

    Attributes:
        id (Union[Unset, int]): Unique data source item identifier.
        fields (Union[None, Unset, list[Union['DataSourceColorThemeItemField', 'DataSourceFontItemField',
            'DataSourceImageItemField', 'DataSourceLanguageItemField', 'DataSourceNumberItemField',
            'DataSourceReferenceItemField', 'DataSourceTextItemField']]]): The fields of the data source item.
    """

    id: Unset | int = UNSET
    fields: (
        None
        | Unset
        | list[
            Union[
                "DataSourceColorThemeItemField",
                "DataSourceFontItemField",
                "DataSourceImageItemField",
                "DataSourceLanguageItemField",
                "DataSourceNumberItemField",
                "DataSourceReferenceItemField",
                "DataSourceTextItemField",
            ]
        ]
    ) = UNSET

    def to_dict(self) -> dict[str, Any]:
        # Top-level imports are used to avoid inline imports (PLC0415)

        id = self.id

        fields: None | Unset | list[dict[str, Any]]
        if isinstance(self.fields, Unset):
            fields = UNSET
        elif isinstance(self.fields, list):
            fields = []
            for fields_type_0_item_data in self.fields:
                fields_type_0_item: dict[str, Any]
                if isinstance(
                    fields_type_0_item_data,
                    (
                        DataSourceTextItemField,
                        DataSourceNumberItemField,
                        DataSourceReferenceItemField,
                        DataSourceImageItemField,
                        DataSourceLanguageItemField,
                        DataSourceFontItemField,
                    ),
                ):
                    fields_type_0_item = fields_type_0_item_data.to_dict()
                else:
                    fields_type_0_item = fields_type_0_item_data.to_dict()

                fields.append(fields_type_0_item)

        else:
            fields = self.fields

        field_dict: dict[str, Any] = {}

        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if fields is not UNSET:
            field_dict["fields"] = fields

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        # Top-level imports are used to avoid inline imports (PLC0415)

        d = dict(src_dict)
        id = d.pop("id", UNSET)

        def _parse_fields(
            data: object,
        ) -> (
            None
            | Unset
            | list[
                Union[
                    "DataSourceColorThemeItemField",
                    "DataSourceFontItemField",
                    "DataSourceImageItemField",
                    "DataSourceLanguageItemField",
                    "DataSourceNumberItemField",
                    "DataSourceReferenceItemField",
                    "DataSourceTextItemField",
                ]
            ]
        ):
            """Create an instance from a dictionary."""
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError
                fields_type_0 = []
                _fields_type_0 = data
                for fields_type_0_item_data in _fields_type_0:

                    def _parse_fields_type_0_item(
                        data: object,
                    ) -> Union[
                        "DataSourceColorThemeItemField",
                        "DataSourceFontItemField",
                        "DataSourceImageItemField",
                        "DataSourceLanguageItemField",
                        "DataSourceNumberItemField",
                        "DataSourceReferenceItemField",
                        "DataSourceTextItemField",
                    ]:
                        try:
                            if not isinstance(data, dict):
                                raise TypeError
                            fields_type_0_item_type_0 = (
                                DataSourceTextItemField.from_dict(data)
                            )

                            return fields_type_0_item_type_0
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError
                            fields_type_0_item_type_1 = (
                                DataSourceNumberItemField.from_dict(data)
                            )

                            return fields_type_0_item_type_1
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError
                            fields_type_0_item_type_2 = (
                                DataSourceReferenceItemField.from_dict(data)
                            )

                            return fields_type_0_item_type_2
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError
                            fields_type_0_item_type_3 = (
                                DataSourceImageItemField.from_dict(data)
                            )

                            return fields_type_0_item_type_3
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError
                            fields_type_0_item_type_4 = (
                                DataSourceLanguageItemField.from_dict(data)
                            )

                            return fields_type_0_item_type_4
                        except:  # noqa: E722
                            pass
                        try:
                            if not isinstance(data, dict):
                                raise TypeError
                            fields_type_0_item_type_5 = (
                                DataSourceFontItemField.from_dict(data)
                            )

                            return fields_type_0_item_type_5
                        except:  # noqa: E722
                            pass
                        if not isinstance(data, dict):
                            raise TypeError
                        fields_type_0_item_type_6 = (
                            DataSourceColorThemeItemField.from_dict(data)
                        )

                        return fields_type_0_item_type_6

                    fields_type_0_item = _parse_fields_type_0_item(
                        fields_type_0_item_data
                    )

                    fields_type_0.append(fields_type_0_item)

                return fields_type_0
            except:  # noqa: E722
                pass
            return cast(
                "None | Unset | list[DataSourceColorThemeItemField | DataSourceFontItemField | DataSourceImageItemField | DataSourceLanguageItemField | DataSourceNumberItemField | DataSourceReferenceItemField | DataSourceTextItemField]",
                data,
            )

        fields = _parse_fields(d.pop("fields", UNSET))

        data_source_item = cls(
            id=id,
            fields=fields,
        )

        return data_source_item



from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.models.folder_state import FolderState
from templafy.types import UNSET, Unset

T = TypeVar("T", bound="Folder")


@_attrs_define
class Folder:
    """Attributes:
    id (int): Unique folder identifier
    library_id (int): Unique library identifier
    name (str): Display name
    navigation_path (str): Hierarchical path in lowercase based on the location of a folder. E.g.
        "folder-a/folder-b" when the location is "Folder A > Folder B"
    modified_at (str): Date and time in ISO 8601 format of when the folder was last modified
    state (FolderState): The current state of the folder
    parent_id (Union[None, Unset, int]): Unique identifier for the parent folder. The root folder does not have a
        parent folder identifier
    """

    id: int
    library_id: int
    name: str
    navigation_path: str
    modified_at: str
    state: FolderState
    parent_id: None | Unset | int = UNSET

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        library_id = self.library_id

        name = self.name

        navigation_path = self.navigation_path

        modified_at = self.modified_at

        state = self.state.value

        parent_id: None | Unset | int
        if isinstance(self.parent_id, Unset):
            parent_id = UNSET
        else:
            parent_id = self.parent_id

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "id": id,
                "libraryId": library_id,
                "name": name,
                "navigationPath": navigation_path,
                "modifiedAt": modified_at,
                "state": state,
            }
        )
        if parent_id is not UNSET:
            field_dict["parentId"] = parent_id

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)
        id = d.pop("id")

        library_id = d.pop("libraryId")

        name = d.pop("name")

        navigation_path = d.pop("navigationPath")

        modified_at = d.pop("modifiedAt")

        state = FolderState(d.pop("state"))

        def _parse_parent_id(data: object) -> None | Unset | int:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | int", data)

        parent_id = _parse_parent_id(d.pop("parentId", UNSET))

        folder = cls(
            id=id,
            library_id=library_id,
            name=name,
            navigation_path=navigation_path,
            modified_at=modified_at,
            state=state,
            parent_id=parent_id,
        )

        return folder



"""Contains all the data models used in inputs/outputs"""

from .asset_file_state_with_previews import AssetFileStateWithPreviews
from .asset_file_state_without_previews import AssetFileStateWithoutPreviews
from .asset_state_without_file import AssetStateWithoutFile
from .color_theme_field_schema import ColorThemeFieldSchema
from .conflict_problem_details import ConflictProblemDetails
from .create_color_theme_data_source_item_field_request import (
    CreateColorThemeDataSourceItemFieldRequest,
)
from .create_data_source_field_schema_request import CreateDataSourceFieldSchemaRequest
from .create_data_source_item_field_request import CreateDataSourceItemFieldRequest
from .create_data_source_item_request import CreateDataSourceItemRequest
from .create_data_source_request import CreateDataSourceRequest
from .create_folder_request import CreateFolderRequest
from .create_font_data_source_item_field_request import (
    CreateFontDataSourceItemFieldRequest,
)
from .create_image_data_source_item_field_request import (
    CreateImageDataSourceItemFieldRequest,
)
from .create_image_field_schema_request import CreateImageFieldSchemaRequest
from .create_link_request import CreateLinkRequest
from .create_number_data_source_item_field_request import (
    CreateNumberDataSourceItemFieldRequest,
)
from .create_number_field_schema_request import CreateNumberFieldSchemaRequest
from .create_reference_data_source_item_field_request import (
    CreateReferenceDataSourceItemFieldRequest,
)
from .create_reference_field_schema_request import CreateReferenceFieldSchemaRequest
from .create_text_data_source_item_field_request import (
    CreateTextDataSourceItemFieldRequest,
)
from .create_text_field_schema_request import CreateTextFieldSchemaRequest
from .data_source import DataSource
from .data_source_color_theme_item_field import DataSourceColorThemeItemField
from .data_source_field_schema import DataSourceFieldSchema
from .data_source_font_item_field import DataSourceFontItemField
from .data_source_image_item_field import DataSourceImageItemField
from .data_source_item import DataSourceItem
from .data_source_item_field import DataSourceItemField
from .data_source_language_item_field import DataSourceLanguageItemField
from .data_source_number_item_field import DataSourceNumberItemField
from .data_source_object_locked_problem_details import (
    DataSourceObjectLockedProblemDetails,
)
from .data_source_reference_item_field import DataSourceReferenceItemField
from .data_source_text_item_field import DataSourceTextItemField
from .dependency import Dependency
from .dimensions import Dimensions
from .document import Document
from .document_details import DocumentDetails
from .email_element import EmailElement
from .email_element_details import EmailElementDetails
from .folder import Folder
from .folder_details import FolderDetails
from .folder_state import FolderState
from .font_field_schema import FontFieldSchema
from .generate_file_request import GenerateFileRequest
from .generate_text_element_file_request import GenerateTextElementFileRequest
from .generated_file import GeneratedFile
from .generated_text_element_file import GeneratedTextElementFile
from .image import Image
from .image_details import ImageDetails
from .image_field_schema import ImageFieldSchema
from .language_field_schema import LanguageFieldSchema
from .library import Library
from .library_details import LibraryDetails
from .library_type import LibraryType
from .link import Link
from .link_details import LinkDetails
from .lock_reason import LockReason
from .not_found_problem_details import NotFoundProblemDetails
from .number_field_schema import NumberFieldSchema
from .patch_color_theme_data_source_item_field_request import (
    PatchColorThemeDataSourceItemFieldRequest,
)
from .patch_data_source_field_request import PatchDataSourceFieldRequest
from .patch_data_source_item_field_request import PatchDataSourceItemFieldRequest
from .patch_data_source_item_request import PatchDataSourceItemRequest
from .patch_font_data_source_item_field_request import (
    PatchFontDataSourceItemFieldRequest,
)
from .patch_image_data_source_item_field_request import (
    PatchImageDataSourceItemFieldRequest,
)
from .patch_libraries_space_id_documents_assets_asset_id_body import (
    PatchLibrariesSpaceIdDocumentsAssetsAssetIdBody,
)
from .patch_libraries_space_id_email_elements_assets_asset_id_body import (
    PatchLibrariesSpaceIdEmailElementsAssetsAssetIdBody,
)
from .patch_libraries_space_id_images_assets_asset_id_body import (
    PatchLibrariesSpaceIdImagesAssetsAssetIdBody,
)
from .patch_libraries_space_id_pdfs_assets_asset_id_body import (
    PatchLibrariesSpaceIdPdfsAssetsAssetIdBody,
)
from .patch_libraries_space_id_presentations_assets_asset_id_body import (
    PatchLibrariesSpaceIdPresentationsAssetsAssetIdBody,
)
from .patch_libraries_space_id_slide_elements_assets_asset_id_body import (
    PatchLibrariesSpaceIdSlideElementsAssetsAssetIdBody,
)
from .patch_libraries_space_id_slides_assets_asset_id_body import (
    PatchLibrariesSpaceIdSlidesAssetsAssetIdBody,
)
from .patch_libraries_space_id_spreadsheets_assets_asset_id_body import (
    PatchLibrariesSpaceIdSpreadsheetsAssetsAssetIdBody,
)
from .patch_libraries_space_id_text_elements_assets_asset_id_body import (
    PatchLibrariesSpaceIdTextElementsAssetsAssetIdBody,
)
from .patch_number_data_source_item_field_request import (
    PatchNumberDataSourceItemFieldRequest,
)
from .patch_reference_data_source_item_field_request import (
    PatchReferenceDataSourceItemFieldRequest,
)
from .patch_text_data_source_item_field_request import (
    PatchTextDataSourceItemFieldRequest,
)
from .pdf import Pdf
from .pdf_details import PdfDetails
from .post_libraries_space_id_documents_folders_folder_id_assets_body import (
    PostLibrariesSpaceIdDocumentsFoldersFolderIdAssetsBody,
)
from .post_libraries_space_id_email_elements_folders_folder_id_assets_body import (
    PostLibrariesSpaceIdEmailElementsFoldersFolderIdAssetsBody,
)
from .post_libraries_space_id_images_folders_folder_id_assets_body import (
    PostLibrariesSpaceIdImagesFoldersFolderIdAssetsBody,
)
from .post_libraries_space_id_pdfs_folders_folder_id_assets_body import (
    PostLibrariesSpaceIdPdfsFoldersFolderIdAssetsBody,
)
from .post_libraries_space_id_presentations_folders_folder_id_assets_body import (
    PostLibrariesSpaceIdPresentationsFoldersFolderIdAssetsBody,
)
from .post_libraries_space_id_slide_elements_folders_folder_id_assets_body import (
    PostLibrariesSpaceIdSlideElementsFoldersFolderIdAssetsBody,
)
from .post_libraries_space_id_slides_folders_folder_id_assets_body import (
    PostLibrariesSpaceIdSlidesFoldersFolderIdAssetsBody,
)
from .post_libraries_space_id_spreadsheets_folders_folder_id_assets_body import (
    PostLibrariesSpaceIdSpreadsheetsFoldersFolderIdAssetsBody,
)
from .post_libraries_space_id_text_elements_folders_folder_id_assets_body import (
    PostLibrariesSpaceIdTextElementsFoldersFolderIdAssetsBody,
)
from .presentation_base import PresentationBase
from .presentation_details_base import PresentationDetailsBase
from .reference_field_schema import ReferenceFieldSchema
from .slide_base import SlideBase
from .slide_details_base import SlideDetailsBase
from .slide_element_base import SlideElementBase
from .slide_element_details_base import SlideElementDetailsBase
from .source_entity_type import SourceEntityType
from .space import Space
from .spreadsheet import Spreadsheet
from .spreadsheet_details import SpreadsheetDetails
from .text_element import TextElement
from .text_element_details import TextElementDetails
from .text_field_schema import TextFieldSchema
from .update_color_theme_data_source_item_field_request import (
    UpdateColorThemeDataSourceItemFieldRequest,
)
from .update_data_source_item_field_request import UpdateDataSourceItemFieldRequest
from .update_data_source_request import UpdateDataSourceRequest
from .update_folder_request import UpdateFolderRequest
from .update_font_data_source_item_field_request import (
    UpdateFontDataSourceItemFieldRequest,
)
from .update_image_data_source_item_field_request import (
    UpdateImageDataSourceItemFieldRequest,
)
from .update_link_request import UpdateLinkRequest
from .update_number_data_source_item_field_request import (
    UpdateNumberDataSourceItemFieldRequest,
)
from .update_reference_data_source_item_field_request import (
    UpdateReferenceDataSourceItemFieldRequest,
)
from .update_text_data_source_item_field_request import (
    UpdateTextDataSourceItemFieldRequest,
)
from .validation_problem_details import ValidationProblemDetails
from .validation_problem_details_errors_type_0 import (
    ValidationProblemDetailsErrorsType0,
)

__all__ = (
    "AssetFileStateWithPreviews",
    "AssetFileStateWithoutPreviews",
    "AssetStateWithoutFile",
    "ColorThemeFieldSchema",
    "ConflictProblemDetails",
    "CreateColorThemeDataSourceItemFieldRequest",
    "CreateDataSourceFieldSchemaRequest",
    "CreateDataSourceItemFieldRequest",
    "CreateDataSourceItemRequest",
    "CreateDataSourceRequest",
    "CreateFolderRequest",
    "CreateFontDataSourceItemFieldRequest",
    "CreateImageDataSourceItemFieldRequest",
    "CreateImageFieldSchemaRequest",
    "CreateLinkRequest",
    "CreateNumberDataSourceItemFieldRequest",
    "CreateNumberFieldSchemaRequest",
    "CreateReferenceDataSourceItemFieldRequest",
    "CreateReferenceFieldSchemaRequest",
    "CreateTextDataSourceItemFieldRequest",
    "CreateTextFieldSchemaRequest",
    "DataSource",
    "DataSourceColorThemeItemField",
    "DataSourceFieldSchema",
    "DataSourceFontItemField",
    "DataSourceImageItemField",
    "DataSourceItem",
    "DataSourceItemField",
    "DataSourceLanguageItemField",
    "DataSourceNumberItemField",
    "DataSourceObjectLockedProblemDetails",
    "DataSourceReferenceItemField",
    "DataSourceTextItemField",
    "Dependency",
    "Dimensions",
    "Document",
    "DocumentDetails",
    "EmailElement",
    "EmailElementDetails",
    "Folder",
    "FolderDetails",
    "FolderState",
    "FontFieldSchema",
    "GenerateFileRequest",
    "GenerateTextElementFileRequest",
    "GeneratedFile",
    "GeneratedTextElementFile",
    "Image",
    "ImageDetails",
    "ImageFieldSchema",
    "LanguageFieldSchema",
    "Library",
    "LibraryDetails",
    "LibraryType",
    "Link",
    "LinkDetails",
    "LockReason",
    "NotFoundProblemDetails",
    "NumberFieldSchema",
    "PatchColorThemeDataSourceItemFieldRequest",
    "PatchDataSourceFieldRequest",
    "PatchDataSourceItemFieldRequest",
    "PatchDataSourceItemRequest",
    "PatchFontDataSourceItemFieldRequest",
    "PatchImageDataSourceItemFieldRequest",
    "PatchLibrariesSpaceIdDocumentsAssetsAssetIdBody",
    "PatchLibrariesSpaceIdEmailElementsAssetsAssetIdBody",
    "PatchLibrariesSpaceIdImagesAssetsAssetIdBody",
    "PatchLibrariesSpaceIdPdfsAssetsAssetIdBody",
    "PatchLibrariesSpaceIdPresentationsAssetsAssetIdBody",
    "PatchLibrariesSpaceIdSlideElementsAssetsAssetIdBody",
    "PatchLibrariesSpaceIdSlidesAssetsAssetIdBody",
    "PatchLibrariesSpaceIdSpreadsheetsAssetsAssetIdBody",
    "PatchLibrariesSpaceIdTextElementsAssetsAssetIdBody",
    "PatchNumberDataSourceItemFieldRequest",
    "PatchReferenceDataSourceItemFieldRequest",
    "PatchTextDataSourceItemFieldRequest",
    "Pdf",
    "PdfDetails",
    "PostLibrariesSpaceIdDocumentsFoldersFolderIdAssetsBody",
    "PostLibrariesSpaceIdEmailElementsFoldersFolderIdAssetsBody",
    "PostLibrariesSpaceIdImagesFoldersFolderIdAssetsBody",
    "PostLibrariesSpaceIdPdfsFoldersFolderIdAssetsBody",
    "PostLibrariesSpaceIdPresentationsFoldersFolderIdAssetsBody",
    "PostLibrariesSpaceIdSlideElementsFoldersFolderIdAssetsBody",
    "PostLibrariesSpaceIdSlidesFoldersFolderIdAssetsBody",
    "PostLibrariesSpaceIdSpreadsheetsFoldersFolderIdAssetsBody",
    "PostLibrariesSpaceIdTextElementsFoldersFolderIdAssetsBody",
    "PresentationBase",
    "PresentationDetailsBase",
    "ReferenceFieldSchema",
    "SlideBase",
    "SlideDetailsBase",
    "SlideElementBase",
    "SlideElementDetailsBase",
    "SourceEntityType",
    "Space",
    "Spreadsheet",
    "SpreadsheetDetails",
    "TextElement",
    "TextElementDetails",
    "TextFieldSchema",
    "UpdateColorThemeDataSourceItemFieldRequest",
    "UpdateDataSourceItemFieldRequest",
    "UpdateDataSourceRequest",
    "UpdateFolderRequest",
    "UpdateFontDataSourceItemFieldRequest",
    "UpdateImageDataSourceItemFieldRequest",
    "UpdateLinkRequest",
    "UpdateNumberDataSourceItemFieldRequest",
    "UpdateReferenceDataSourceItemFieldRequest",
    "UpdateTextDataSourceItemFieldRequest",
    "ValidationProblemDetails",
    "ValidationProblemDetailsErrorsType0",
)



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.types import UNSET, Unset

T = TypeVar("T", bound="GenerateFileRequest")


@_attrs_define
class GenerateFileRequest:
    """The request model to generate a file.

    Example:
        {'email': 'templafy@templafy.com', 'data': {'Language': 'en-us'}, 'includePdf': True}

    Attributes:
        email (str): Email to be used for identification.
        data (Union[Unset, Any]): Data to be used during the file generation.
        include_pdf (Union[Unset, bool]): Specifies whether a PDF export of the file should be included in the response.
    """

    email: str
    data: Unset | Any = UNSET
    include_pdf: Unset | bool = UNSET

    def to_dict(self) -> dict[str, Any]:
        email = self.email

        data = self.data

        include_pdf = self.include_pdf

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "email": email,
            }
        )
        if data is not UNSET:
            field_dict["data"] = data
        if include_pdf is not UNSET:
            field_dict["includePdf"] = include_pdf

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)
        email = d.pop("email")

        data = d.pop("data", UNSET)

        include_pdf = d.pop("includePdf", UNSET)

        generate_file_request = cls(
            email=email,
            data=data,
            include_pdf=include_pdf,
        )

        return generate_file_request



from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.types import UNSET, Unset

T = TypeVar("T", bound="CreateLinkRequest")


@_attrs_define
class CreateLinkRequest:
    """The request model to create a link asset

    Attributes:
        name (str): Display name
        url (str): A reference to the web resource. HTTP and HTTPS are supported
        description (Union[None, Unset, str]): Describing intended usage of the asset
        tags (Union[None, Unset, list[str]]): Tags should describe the content of the asset making it easier for a user
            to locate it
        external_data (Union[None, Unset, str]): External data which can be attached for future reference
    """

    name: str
    url: str
    description: None | Unset | str = UNSET
    tags: None | Unset | list[str] = UNSET
    external_data: None | Unset | str = UNSET

    def to_dict(self) -> dict[str, Any]:
        """Convert the object to a dictionary."""
        name = self.name

        url = self.url

        description: None | Unset | str
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        tags: None | Unset | list[str]
        if isinstance(self.tags, Unset):
            tags = UNSET
        elif isinstance(self.tags, list):
            tags = self.tags

        else:
            tags = self.tags

        external_data: None | Unset | str
        if isinstance(self.external_data, Unset):
            external_data = UNSET
        else:
            external_data = self.external_data

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "name": name,
                "url": url,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if tags is not UNSET:
            field_dict["tags"] = tags
        if external_data is not UNSET:
            field_dict["externalData"] = external_data

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        """Create an instance from a dictionary."""
        d = dict(src_dict)
        name = d.pop("name")

        url = d.pop("url")

        def _parse_description(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_tags(data: object) -> None | Unset | list[str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError
                tags_type_0 = cast("list[str]", data)

                return tags_type_0
            except:  # noqa: E722
                pass
            return cast("None | Unset | list[str]", data)

        tags = _parse_tags(d.pop("tags", UNSET))

        def _parse_external_data(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        external_data = _parse_external_data(d.pop("externalData", UNSET))

        create_link_request = cls(
            name=name,
            url=url,
            description=description,
            tags=tags,
            external_data=external_data,
        )

        return create_link_request



from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.models.asset_file_state_without_previews import (
    AssetFileStateWithoutPreviews,
)
from templafy.types import UNSET, Unset

T = TypeVar("T", bound="Spreadsheet")


@_attrs_define
class Spreadsheet:
    """Attributes:
    id (int): Unique asset identifier
    folder_id (int): Unique folder identifier
    name (str): Display name
    description (str):
    tags (list[str]):
    file_size (int): File size in bytes
    checksum (str): MD5 checksum of the bytes
    file_extension (str): Suffix to the name of the file
    navigation_path (str): Hierarchical path in lowercase based on the location of a spreadsheet. E.g.
        "folder-a/folder-b/_my-spreadsheet" when the location is "Folder A > Folder B > My Spreadsheet"
    modified_at (str): Date and time in ISO 8601 format of when the asset was last modified
    asset_state (AssetFileStateWithoutPreviews): The current state of the asset
    external_data (Union[None, Unset, str]): External data which can be attached for future reference
    """

    id: int
    folder_id: int
    name: str
    description: str
    tags: list[str]
    file_size: int
    checksum: str
    file_extension: str
    navigation_path: str
    modified_at: str
    asset_state: AssetFileStateWithoutPreviews
    external_data: None | Unset | str = UNSET

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        folder_id = self.folder_id

        name = self.name

        description = self.description

        tags = self.tags

        file_size = self.file_size

        checksum = self.checksum

        file_extension = self.file_extension

        navigation_path = self.navigation_path

        modified_at = self.modified_at

        asset_state = self.asset_state.value

        external_data: None | Unset | str
        if isinstance(self.external_data, Unset):
            external_data = UNSET
        else:
            external_data = self.external_data

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "id": id,
                "folderId": folder_id,
                "name": name,
                "description": description,
                "tags": tags,
                "fileSize": file_size,
                "checksum": checksum,
                "fileExtension": file_extension,
                "navigationPath": navigation_path,
                "modifiedAt": modified_at,
                "assetState": asset_state,
            }
        )
        if external_data is not UNSET:
            field_dict["externalData"] = external_data

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)
        id = d.pop("id")

        folder_id = d.pop("folderId")

        name = d.pop("name")

        description = d.pop("description")

        tags = cast("list[str]", d.pop("tags"))

        file_size = d.pop("fileSize")

        checksum = d.pop("checksum")

        file_extension = d.pop("fileExtension")

        navigation_path = d.pop("navigationPath")

        modified_at = d.pop("modifiedAt")

        asset_state = AssetFileStateWithoutPreviews(d.pop("assetState"))

        def _parse_external_data(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        external_data = _parse_external_data(d.pop("externalData", UNSET))

        spreadsheet = cls(
            id=id,
            folder_id=folder_id,
            name=name,
            description=description,
            tags=tags,
            file_size=file_size,
            checksum=checksum,
            file_extension=file_extension,
            navigation_path=navigation_path,
            modified_at=modified_at,
            asset_state=asset_state,
            external_data=external_data,
        )

        return spreadsheet



from http import HTTPStatus
from typing import Any, cast

import httpx

from templafy import errors
from templafy.client import AuthenticatedClient, Client
from templafy.models.space import Space
from templafy.types import Response


def _get_kwargs() -> dict[str, Any]:
    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/spaces",
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Any | list["Space"] | None:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = Space.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200

    if response.status_code == 401:
        response_401 = cast("Any", None)
        return response_401

    if response.status_code == 403:
        response_403 = cast("Any", None)
        return response_403

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[Any | list["Space"]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
) -> Response[Any | list["Space"]]:
    """Lists all existing active spaces.

     Returns a list of all active spaces.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, list['Space']]]
    """

    kwargs = _get_kwargs()

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
) -> Any | list["Space"] | None:
    """Lists all existing active spaces.

     Returns a list of all active spaces.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, list['Space']]
    """

    return sync_detailed(
        client=client,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
) -> Response[Any | list["Space"]]:
    """Lists all existing active spaces.

     Returns a list of all active spaces.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, list['Space']]]
    """

    kwargs = _get_kwargs()

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
) -> Any | list["Space"] | None:
    """Lists all existing active spaces.

     Returns a list of all active spaces.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, list['Space']]
    """

    return (
        await asyncio_detailed(
            client=client,
        )
    ).parsed



from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.types import UNSET, Unset

T = TypeVar("T", bound="UpdateDataSourceRequest")


@_attrs_define
class UpdateDataSourceRequest:
    """Example:
        {'description': 'This is an updated description'}

    Attributes:
        description (Union[None, Unset, str]): Data source description. If null, description will be removed.
    """

    description: None | Unset | str = UNSET

    def to_dict(self) -> dict[str, Any]:
        description: None | Unset | str
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        field_dict: dict[str, Any] = {}

        field_dict.update({})
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)

        def _parse_description(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        description = _parse_description(d.pop("description", UNSET))

        update_data_source_request = cls(
            description=description,
        )

        return update_data_source_request



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Self

T = TypeVar("T", bound="CreateColorThemeDataSourceItemFieldRequest")


@_attrs_define
class CreateColorThemeDataSourceItemFieldRequest:
    """Example:
        {'type': 'colorTheme', 'dataSourceFieldId': 5, 'xmlValue': '<a:clrScheme
            xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main" name="Templafy_New"><a:dk1><a:srgbClr
            val="0078FF"/></a:dk1><a:lt1><a:sysClr val="window" lastClr="FFFFFF"/></a:lt1><a:dk2><a:srgbClr
            val="000000"/></a:dk2><a:lt2><a:srgbClr val="E5E5E5"/></a:lt2><a:accent1><a:srgbClr
            val="0078FF"/></a:accent1><a:accent2><a:srgbClr val="575757"/></a:accent2><a:accent3><a:srgbClr
            val="12AA96"/></a:accent3><a:accent4><a:srgbClr val="15436B"/></a:accent4><a:accent5><a:srgbClr
            val="D44849"/></a:accent5><a:accent6><a:srgbClr val="7F7F7F"/></a:accent6><a:hlink><a:srgbClr
            val="0078FF"/></a:hlink><a:folHlink><a:srgbClr val="55CBFF"/></a:folHlink></a:clrScheme>'}

    Attributes:
        type_ (str): Data source item field type.
        data_source_field_id (int): Data source field identifier.
        xml_value (str): The value of the field based on the schema
            http://schemas.openxmlformats.org/drawingml/2006/main. Max length is 3500 characters.
    """

    type_: str
    data_source_field_id: int
    xml_value: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        """Convert the object to a dictionary."""
        type_ = self.type_

        data_source_field_id = self.data_source_field_id

        xml_value = self.xml_value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type_,
                "dataSourceFieldId": data_source_field_id,
                "xmlValue": xml_value,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        """Create an instance from a dictionary."""
        d = dict(src_dict)
        type_ = d.pop("type")

        data_source_field_id = d.pop("dataSourceFieldId")

        xml_value = d.pop("xmlValue")

        create_color_theme_data_source_item_field_request = cls(
            type_=type_,
            data_source_field_id=data_source_field_id,
            xml_value=xml_value,
        )

        create_color_theme_data_source_item_field_request.additional_properties = d
        return create_color_theme_data_source_item_field_request

    @property
    def additional_keys(self) -> list[str]:
        """Return the list of additional property keys."""
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, cast

import httpx

from templafy import errors
from templafy.client import AuthenticatedClient, Client
from templafy.models.not_found_problem_details import NotFoundProblemDetails
from templafy.models.update_data_source_request import UpdateDataSourceRequest
from templafy.models.validation_problem_details import ValidationProblemDetails
from templafy.types import Response


def _get_kwargs(
    id: int,
    *,
    body: UpdateDataSourceRequest,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "patch",
        "url": f"/data-sources/{id}",
    }

    if isinstance(body, UpdateDataSourceRequest):
        _kwargs["json"] = body.to_dict()

        headers["Content-Type"] = "application/json-patch+json"
    if isinstance(body, UpdateDataSourceRequest):
        _kwargs["json"] = body.to_dict()

        headers["Content-Type"] = "application/json"
    if isinstance(body, UpdateDataSourceRequest):
        _kwargs["json"] = body.to_dict()

        headers["Content-Type"] = "application/*+json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Any | NotFoundProblemDetails | ValidationProblemDetails | None:
    if response.status_code == 204:
        response_204 = cast("Any", None)
        return response_204

    if response.status_code == 400:
        response_400 = ValidationProblemDetails.from_dict(response.json())

        return response_400

    if response.status_code == 401:
        response_401 = cast("Any", None)
        return response_401

    if response.status_code == 403:
        response_403 = cast("Any", None)
        return response_403

    if response.status_code == 404:
        response_404 = NotFoundProblemDetails.from_dict(response.json())

        return response_404

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[Any | NotFoundProblemDetails | ValidationProblemDetails]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: int,
    *,
    client: AuthenticatedClient,
    body: UpdateDataSourceRequest,
) -> Response[Any | NotFoundProblemDetails | ValidationProblemDetails]:
    """Updates an existing data source.

     This is a PATCH operation. Any fields not included in the request will remain unchanged on the
    server.

    Args:
        id (int):
        body (UpdateDataSourceRequest):  Example: {'description': 'This is an updated
            description'}.
        body (UpdateDataSourceRequest):  Example: {'description': 'This is an updated
            description'}.
        body (UpdateDataSourceRequest):  Example: {'description': 'This is an updated
            description'}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, NotFoundProblemDetails, ValidationProblemDetails]]
    """

    kwargs = _get_kwargs(
        id=id,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    id: int,
    *,
    client: AuthenticatedClient,
    body: UpdateDataSourceRequest,
) -> Any | NotFoundProblemDetails | ValidationProblemDetails | None:
    """Updates an existing data source.

     This is a PATCH operation. Any fields not included in the request will remain unchanged on the
    server.

    Args:
        id (int):
        body (UpdateDataSourceRequest):  Example: {'description': 'This is an updated
            description'}.
        body (UpdateDataSourceRequest):  Example: {'description': 'This is an updated
            description'}.
        body (UpdateDataSourceRequest):  Example: {'description': 'This is an updated
            description'}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, NotFoundProblemDetails, ValidationProblemDetails]
    """

    return sync_detailed(
        id=id,
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    id: int,
    *,
    client: AuthenticatedClient,
    body: UpdateDataSourceRequest,
) -> Response[Any | NotFoundProblemDetails | ValidationProblemDetails]:
    """Updates an existing data source.

     This is a PATCH operation. Any fields not included in the request will remain unchanged on the
    server.

    Args:
        id (int):
        body (UpdateDataSourceRequest):  Example: {'description': 'This is an updated
            description'}.
        body (UpdateDataSourceRequest):  Example: {'description': 'This is an updated
            description'}.
        body (UpdateDataSourceRequest):  Example: {'description': 'This is an updated
            description'}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, NotFoundProblemDetails, ValidationProblemDetails]]
    """

    kwargs = _get_kwargs(
        id=id,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    id: int,
    *,
    client: AuthenticatedClient,
    body: UpdateDataSourceRequest,
) -> Any | NotFoundProblemDetails | ValidationProblemDetails | None:
    """Updates an existing data source.

     This is a PATCH operation. Any fields not included in the request will remain unchanged on the
    server.

    Args:
        id (int):
        body (UpdateDataSourceRequest):  Example: {'description': 'This is an updated
            description'}.
        body (UpdateDataSourceRequest):  Example: {'description': 'This is an updated
            description'}.
        body (UpdateDataSourceRequest):  Example: {'description': 'This is an updated
            description'}.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, NotFoundProblemDetails, ValidationProblemDetails]
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            body=body,
        )
    ).parsed



from enum import Enum


class AssetFileStateWithPreviews(str, Enum):
    """Enumeration of possible file processing states for an asset that
    includes previews.

    Values represent server-side lifecycle stages for an asset's file and
    its generated previews.
    """

    DELETING = "deleting"
    GENERATINGPREVIEWS = "generatingPreviews"
    GENERATINGPREVIEWSFAILED = "generatingPreviewsFailed"
    PROCESSING = "processing"
    PROCESSINGFAILED = "processingFailed"
    READY = "ready"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Self

T = TypeVar("T", bound="UpdateReferenceDataSourceItemFieldRequest")


@_attrs_define
class UpdateReferenceDataSourceItemFieldRequest:
    """Example:
        {'type': 'reference', 'dataSourceItemId': '638247997437572264'}

    Attributes:
        type_ (str): Data source item field type.
        data_source_item_id (int): The identifier of the data source item to be referenced.
    """

    type_: str
    data_source_item_id: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        data_source_item_id = self.data_source_item_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type_,
                "dataSourceItemId": data_source_item_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)
        type_ = d.pop("type")

        data_source_item_id = d.pop("dataSourceItemId")

        update_reference_data_source_item_field_request = cls(
            type_=type_,
            data_source_item_id=data_source_item_id,
        )

        update_reference_data_source_item_field_request.additional_properties = d
        return update_reference_data_source_item_field_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.types import UNSET, Unset

if TYPE_CHECKING:
    from templafy.models.color_theme_field_schema import ColorThemeFieldSchema
    from templafy.models.font_field_schema import FontFieldSchema
    from templafy.models.image_field_schema import ImageFieldSchema
    from templafy.models.language_field_schema import LanguageFieldSchema
    from templafy.models.number_field_schema import NumberFieldSchema
    from templafy.models.reference_field_schema import ReferenceFieldSchema
    from templafy.models.text_field_schema import TextFieldSchema

# Top-level runtime imports to satisfy PLC0415
from templafy.models.color_theme_field_schema import ColorThemeFieldSchema
from templafy.models.font_field_schema import FontFieldSchema
from templafy.models.image_field_schema import ImageFieldSchema
from templafy.models.language_field_schema import LanguageFieldSchema
from templafy.models.number_field_schema import NumberFieldSchema
from templafy.models.reference_field_schema import ReferenceFieldSchema
from templafy.models.text_field_schema import TextFieldSchema

T = TypeVar("T", bound="DataSource")


@_attrs_define
class DataSource:
    """Example:
        {'id': '638247997499047080', 'name': 'Cities', 'description': 'Cities in which we have offices', 'fields':
            [{'id': 0, 'name': 'Name', 'type': 'text', 'isRequired': True, 'isLocked': True, 'isMultipleLines': False},
            {'id': 1, 'name': 'History', 'type': 'text', 'isRequired': False, 'isLocked': False, 'isMultipleLines': True,
            'defaultValue': 'The city was established in the year 1652 by Dutch explorers...'}, {'id': 2, 'name':
            'Population', 'type': 'number', 'isRequired': True, 'isLocked': False, 'defaultValue': 100222}, {'id': 3,
            'name': 'Country', 'type': 'reference', 'isRequired': True, 'isLocked': False, 'defaultValue':
            '638247997437572266', 'referenceDataSourceId': '638247997437572264'}, {'id': 4, 'name': 'Flag', 'type': 'image',
            'isRequired': False, 'isLocked': False}, {'id': 5, 'name': 'PreferredLanguage', 'type': 'language',
            'isRequired': False, 'isLocked': False, 'defaultValue': 'German'}, {'id': 6, 'name': 'PreferredFont', 'type':
            'font', 'isRequired': False, 'isLocked': False}, {'id': 7, 'name': 'PreferredColourTheme', 'type': 'colorTheme',
            'isRequired': False, 'isLocked': False}]}

    Attributes:
        id (int): Unique data source identifier.
        name (str): Data source name. It must be unique.
        fields (list[Union['ColorThemeFieldSchema', 'FontFieldSchema', 'ImageFieldSchema', 'LanguageFieldSchema',
            'NumberFieldSchema', 'ReferenceFieldSchema', 'TextFieldSchema']]):
        description (Union[None, Unset, str]): Data source description.
    """

    id: int
    name: str
    fields: list[
        Union[
            "ColorThemeFieldSchema",
            "FontFieldSchema",
            "ImageFieldSchema",
            "LanguageFieldSchema",
            "NumberFieldSchema",
            "ReferenceFieldSchema",
            "TextFieldSchema",
        ]
    ]
    description: None | Unset | str = UNSET

    def to_dict(self) -> dict[str, Any]:
        """Convert the object to a dictionary."""
        # Top-level imports are used to avoid inline imports (PLC0415)

        id = self.id

        name = self.name

        fields = []
        for fields_item_data in self.fields:
            fields_item: dict[str, Any]
            if isinstance(
                fields_item_data,
                (
                    TextFieldSchema,
                    NumberFieldSchema,
                    ReferenceFieldSchema,
                    ImageFieldSchema,
                    LanguageFieldSchema,
                    FontFieldSchema,
                ),
            ):
                fields_item = fields_item_data.to_dict()
            else:
                fields_item = fields_item_data.to_dict()

            fields.append(fields_item)

        description: None | Unset | str
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "id": id,
                "name": name,
                "fields": fields,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        """Create an instance from a dictionary."""
        # Top-level imports are used to avoid inline imports (PLC0415)

        d = dict(src_dict)
        id = d.pop("id")

        name = d.pop("name")

        fields = []
        _fields = d.pop("fields")
        for fields_item_data in _fields:

            def _parse_fields_item(
                data: object,
            ) -> Union[
                "ColorThemeFieldSchema",
                "FontFieldSchema",
                "ImageFieldSchema",
                "LanguageFieldSchema",
                "NumberFieldSchema",
                "ReferenceFieldSchema",
                "TextFieldSchema",
            ]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError
                    fields_item_type_0 = TextFieldSchema.from_dict(data)

                    return fields_item_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError
                    fields_item_type_1 = NumberFieldSchema.from_dict(data)

                    return fields_item_type_1
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError
                    fields_item_type_2 = ReferenceFieldSchema.from_dict(data)

                    return fields_item_type_2
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError
                    fields_item_type_3 = ImageFieldSchema.from_dict(data)

                    return fields_item_type_3
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError
                    fields_item_type_4 = LanguageFieldSchema.from_dict(data)

                    return fields_item_type_4
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError
                    fields_item_type_5 = FontFieldSchema.from_dict(data)

                    return fields_item_type_5
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError
                fields_item_type_6 = ColorThemeFieldSchema.from_dict(data)

                return fields_item_type_6

            fields_item = _parse_fields_item(fields_item_data)

            fields.append(fields_item)

        def _parse_description(data: object) -> None | Unset | str:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast("None | Unset | str", data)

        description = _parse_description(d.pop("description", UNSET))

        data_source = cls(
            id=id,
            name=name,
            fields=fields,
            description=description,
        )

        return data_source



from http import HTTPStatus
from typing import Any, cast

import httpx

from templafy import errors
from templafy.client import AuthenticatedClient, Client
from templafy.models.conflict_problem_details import ConflictProblemDetails
from templafy.models.not_found_problem_details import NotFoundProblemDetails
from templafy.models.patch_libraries_space_id_presentations_assets_asset_id_body import (
    PatchLibrariesSpaceIdPresentationsAssetsAssetIdBody,
)
from templafy.models.validation_problem_details import ValidationProblemDetails
from templafy.types import Response


def _get_kwargs(
    space_id: int,
    asset_id: int,
    *,
    body: PatchLibrariesSpaceIdPresentationsAssetsAssetIdBody,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "patch",
        "url": f"/libraries/{space_id}/presentations/assets/{asset_id}",
    }

    _kwargs["files"] = body.to_multipart()

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> (
    Any
    | ConflictProblemDetails
    | NotFoundProblemDetails
    | ValidationProblemDetails
    | None
):
    if response.status_code == 204:
        response_204 = cast("Any", None)
        return response_204

    if response.status_code == 400:
        response_400 = ValidationProblemDetails.from_dict(response.json())

        return response_400

    if response.status_code == 401:
        response_401 = cast("Any", None)
        return response_401

    if response.status_code == 403:
        response_403 = cast("Any", None)
        return response_403

    if response.status_code == 404:
        response_404 = NotFoundProblemDetails.from_dict(response.json())

        return response_404

    if response.status_code == 409:
        response_409 = ConflictProblemDetails.from_dict(response.json())

        return response_409

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[
    Any | ConflictProblemDetails | NotFoundProblemDetails | ValidationProblemDetails
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    space_id: int,
    asset_id: int,
    *,
    client: AuthenticatedClient,
    body: PatchLibrariesSpaceIdPresentationsAssetsAssetIdBody,
) -> Response[
    Any | ConflictProblemDetails | NotFoundProblemDetails | ValidationProblemDetails
]:
    """Updates the presentation template.

     Only one file can be attached to the request body. The supported file format is .PPTX. Slides are
    limited to the following properties that can be updated: Name, Tags, ExternalData

    Args:
        space_id (int):
        asset_id (int):
        body (PatchLibrariesSpaceIdPresentationsAssetsAssetIdBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, ConflictProblemDetails, NotFoundProblemDetails, ValidationProblemDetails]]
    """

    kwargs = _get_kwargs(
        space_id=space_id,
        asset_id=asset_id,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    space_id: int,
    asset_id: int,
    *,
    client: AuthenticatedClient,
    body: PatchLibrariesSpaceIdPresentationsAssetsAssetIdBody,
) -> (
    Any
    | ConflictProblemDetails
    | NotFoundProblemDetails
    | ValidationProblemDetails
    | None
):
    """Updates the presentation template.

     Only one file can be attached to the request body. The supported file format is .PPTX. Slides are
    limited to the following properties that can be updated: Name, Tags, ExternalData

    Args:
        space_id (int):
        asset_id (int):
        body (PatchLibrariesSpaceIdPresentationsAssetsAssetIdBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, ConflictProblemDetails, NotFoundProblemDetails, ValidationProblemDetails]
    """

    return sync_detailed(
        space_id=space_id,
        asset_id=asset_id,
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    space_id: int,
    asset_id: int,
    *,
    client: AuthenticatedClient,
    body: PatchLibrariesSpaceIdPresentationsAssetsAssetIdBody,
) -> Response[
    Any | ConflictProblemDetails | NotFoundProblemDetails | ValidationProblemDetails
]:
    """Updates the presentation template.

     Only one file can be attached to the request body. The supported file format is .PPTX. Slides are
    limited to the following properties that can be updated: Name, Tags, ExternalData

    Args:
        space_id (int):
        asset_id (int):
        body (PatchLibrariesSpaceIdPresentationsAssetsAssetIdBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, ConflictProblemDetails, NotFoundProblemDetails, ValidationProblemDetails]]
    """

    kwargs = _get_kwargs(
        space_id=space_id,
        asset_id=asset_id,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    space_id: int,
    asset_id: int,
    *,
    client: AuthenticatedClient,
    body: PatchLibrariesSpaceIdPresentationsAssetsAssetIdBody,
) -> (
    Any
    | ConflictProblemDetails
    | NotFoundProblemDetails
    | ValidationProblemDetails
    | None
):
    """Updates the presentation template.

     Only one file can be attached to the request body. The supported file format is .PPTX. Slides are
    limited to the following properties that can be updated: Name, Tags, ExternalData

    Args:
        space_id (int):
        asset_id (int):
        body (PatchLibrariesSpaceIdPresentationsAssetsAssetIdBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, ConflictProblemDetails, NotFoundProblemDetails, ValidationProblemDetails]
    """

    return (
        await asyncio_detailed(
            space_id=space_id,
            asset_id=asset_id,
            client=client,
            body=body,
        )
    ).parsed



from collections.abc import Mapping
from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from typing_extensions import Self

from templafy.models.patch_color_theme_data_source_item_field_request import (
    PatchColorThemeDataSourceItemFieldRequest,
)
from templafy.models.patch_font_data_source_item_field_request import (
    PatchFontDataSourceItemFieldRequest,
)
from templafy.models.patch_image_data_source_item_field_request import (
    PatchImageDataSourceItemFieldRequest,
)
from templafy.models.patch_number_data_source_item_field_request import (
    PatchNumberDataSourceItemFieldRequest,
)
from templafy.models.patch_reference_data_source_item_field_request import (
    PatchReferenceDataSourceItemFieldRequest,
)
from templafy.models.patch_text_data_source_item_field_request import (
    PatchTextDataSourceItemFieldRequest,
)

T = TypeVar("T", bound="PatchDataSourceItemRequest")


@_attrs_define
class PatchDataSourceItemRequest:
    """Attributes:
    fields (list[Union['PatchColorThemeDataSourceItemFieldRequest', 'PatchFontDataSourceItemFieldRequest',
        'PatchImageDataSourceItemFieldRequest', 'PatchNumberDataSourceItemFieldRequest',
        'PatchReferenceDataSourceItemFieldRequest', 'PatchTextDataSourceItemFieldRequest']]): The fields of the data
        source item.
    """

    fields: list[
        Union[
            "PatchColorThemeDataSourceItemFieldRequest",
            "PatchFontDataSourceItemFieldRequest",
            "PatchImageDataSourceItemFieldRequest",
            "PatchNumberDataSourceItemFieldRequest",
            "PatchReferenceDataSourceItemFieldRequest",
            "PatchTextDataSourceItemFieldRequest",
        ]
    ]

    def to_dict(self) -> dict[str, Any]:
        """Return a dictionary representation of the model."""
        fields = []
        for fields_item_data in self.fields:
            fields_item: dict[str, Any]
            if isinstance(
                fields_item_data,
                (
                    PatchTextDataSourceItemFieldRequest,
                    PatchNumberDataSourceItemFieldRequest,
                    PatchImageDataSourceItemFieldRequest,
                    PatchReferenceDataSourceItemFieldRequest,
                    PatchFontDataSourceItemFieldRequest,
                ),
            ):
                fields_item = fields_item_data.to_dict()
            else:
                fields_item = fields_item_data.to_dict()

            fields.append(fields_item)

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "fields": fields,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        # model classes are imported at module level to satisfy linter PLC0415
        """Create a PatchDataSourceItemRequest instance from a mapping/dict.

        Args:
            src_dict: Mapping representation of the model.

        Returns:
            An instantiated PatchDataSourceItemRequest.
        """

        d = dict(src_dict)
        fields = []
        _fields = d.pop("fields")
        for fields_item_data in _fields:

            def _parse_fields_item(
                data: object,
            ) -> Union[
                "PatchColorThemeDataSourceItemFieldRequest",
                "PatchFontDataSourceItemFieldRequest",
                "PatchImageDataSourceItemFieldRequest",
                "PatchNumberDataSourceItemFieldRequest",
                "PatchReferenceDataSourceItemFieldRequest",
                "PatchTextDataSourceItemFieldRequest",
            ]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError
                    fields_item_type_0 = PatchTextDataSourceItemFieldRequest.from_dict(
                        data
                    )

                    return fields_item_type_0
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError
                    fields_item_type_1 = (
                        PatchNumberDataSourceItemFieldRequest.from_dict(data)
                    )

                    return fields_item_type_1
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError
                    fields_item_type_2 = PatchImageDataSourceItemFieldRequest.from_dict(
                        data
                    )

                    return fields_item_type_2
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError
                    fields_item_type_3 = (
                        PatchReferenceDataSourceItemFieldRequest.from_dict(data)
                    )

                    return fields_item_type_3
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError
                    fields_item_type_4 = PatchFontDataSourceItemFieldRequest.from_dict(
                        data
                    )

                    return fields_item_type_4
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError
                fields_item_type_5 = (
                    PatchColorThemeDataSourceItemFieldRequest.from_dict(data)
                )

                return fields_item_type_5

            fields_item = _parse_fields_item(fields_item_data)

            fields.append(fields_item)

        patch_data_source_item_request = cls(
            fields=fields,
        )

        return patch_data_source_item_request



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from typing_extensions import Self

T = TypeVar("T", bound="UpdateDataSourceItemFieldRequest")


@_attrs_define
class UpdateDataSourceItemFieldRequest:
    """Example:
        {'type': 'text', 'value': 'An updated value'}

    Attributes:
        type_ (str): Data source item field type.
    """

    type_: str

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "type": type_,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)
        type_ = d.pop("type")

        update_data_source_item_field_request = cls(
            type_=type_,
        )

        return update_data_source_item_field_request



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from typing_extensions import Self

T = TypeVar("T", bound="GeneratedTextElementFile")


@_attrs_define
class GeneratedTextElementFile:
    """The generated text element file response model.

    Attributes:
        download_url (str): Temporary access URL for generated file.
        file_size (int): File size in bytes.
        checksum (str): MD5 checksum of the bytes.
        mime_type (str): Mime type of the generated file.
        file_extension (str): Suffix to the name of the file.
    """

    download_url: str
    file_size: int
    checksum: str
    mime_type: str
    file_extension: str

    def to_dict(self) -> dict[str, Any]:
        download_url = self.download_url

        file_size = self.file_size

        checksum = self.checksum

        mime_type = self.mime_type

        file_extension = self.file_extension

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "downloadUrl": download_url,
                "fileSize": file_size,
                "checksum": checksum,
                "mimeType": mime_type,
                "fileExtension": file_extension,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls, src_dict: Mapping[str, Any]) -> Self:
        d = dict(src_dict)
        download_url = d.pop("downloadUrl")

        file_size = d.pop("fileSize")

        checksum = d.pop("checksum")

        mime_type = d.pop("mimeType")

        file_extension = d.pop("fileExtension")

        generated_text_element_file = cls(
            download_url=download_url,
            file_size=file_size,
            checksum=checksum,
            mime_type=mime_type,
            file_extension=file_extension,
        )

        return generated_text_element_file
